#!/usr/bin/env python3
#
# Original espota.py by Ivan Grokhotkov:
# https://gist.github.com/igrr/d35ab8446922179dc58c
#
# Modified since 2015-09-18 from Pascal Gollor (https://github.com/pgollor)
# Modified since 2015-11-09 from Hristo Gochkov (https://github.com/me-no-dev)
# Modified since 2016-01-03 from Matthew O'Gorman (https://githumb.com/mogorman)
#
# This script will push an OTA update to the ESP
# use it like: python3 espota.py -i <ESP_IP_address> -I <Host_IP_address> -p <ESP_port> -P <Host_port> [-a password] -f <sketch.bin>
# Or to upload SPIFFS image:
# python3 espota.py -i <ESP_IP_address> -I <Host_IP_address> -p <ESP_port> -P <HOST_port> [-a password] -s -f <spiffs.bin>
#
# Changes
# 2015-09-18:
# - Add option parser.
# - Add logging.
# - Send command to controller to differ between flashing and transmitting SPIFFS image.
#
# Changes
# 2015-11-09:
# - Added digest authentication
# - Enhanced error tracking and reporting
#
# Changes
# 2016-01-03:
# - Added more options to parser.
#

from __future__ import print_function
import socket
import sys
import os
import optparse
import logging
import hashlib
import random

# Commands
FLASH = 0
SPIFFS = 100
AUTH = 200
PROGRESS = False
# update_progress() : Displays or updates a console progress bar
## Accepts a float between 0 and 1. Any int will be converted to a float.
## A value under 0 represents a 'halt'.
## A value at 1 or bigger represents 100%
def update_progress(progress):
  if (PROGRESS):
    barLength = 60 # Modify this to change the length of the progress bar
    status = ""
    if isinstance(progress, int):
      progress = float(progress)
    if not isinstance(progress, float):
      progress = 0
      status = "error: progress var must be float\r\n"
    if progress < 0:
      progress = 0
      status = "Halt...\r\n"
    if progress >= 1:
      progress = 1
      status = "Done...\r\n"
    block = int(round(barLength*progress))
    text = "\rUploading: [{0}] {1}% {2}".format( "="*block + " "*(barLength-block), int(progress*100), status)
    sys.stderr.write(text)
    sys.stderr.flush()
  else:
    sys.stderr.write('.')
    sys.stderr.flush()

def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, command = FLASH):
  # Create a TCP/IP socket
  sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
  server_address = (localAddr, localPort)
  logging.info('Starting on %s:%s', str(server_address[0]), str(server_address[1]))
  try:
    sock.bind(server_address)
    sock.listen(1)
  except Exception:
    logging.error("Listen Failed")
    return 1

  # Check whether Signed Update is used.
  if ( os.path.isfile(filename + '.signed') ):
    filename = filename + '.signed'
    file_check_msg = 'Detected Signed Update. %s will be uploaded instead.' % (filename)
    sys.stderr.write(file_check_msg + '\n')
    sys.stderr.flush()
    logging.info(file_check_msg)
  
  content_size = os.path.getsize(filename)
  f = open(filename,'rb')
  file_md5 = hashlib.md5(f.read()).hexdigest()
  f.close()
  logging.info('Upload size: %d', content_size)
  message = '%d %d %d %s\n' % (command, localPort, content_size, file_md5)

  # Wait for a connection
  logging.info('Sending invitation to: %s', remoteAddr)
  sock2 = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
  remote_address = (remoteAddr, int(remotePort))
  sock2.sendto(message.encode(), remote_address)
  sock2.settimeout(10)
  try:
    data = sock2.recv(128).decode()
  except Exception:
    logging.error('No Answer')
    sock2.close()
    return 1
  if (data != "OK"):
    if(data.startswith('AUTH')):
      nonce = data.split()[1]
      cnonce_text = '%s%u%s%s' % (filename, content_size, file_md5, remoteAddr)
      cnonce = hashlib.md5(cnonce_text.encode()).hexdigest()
      passmd5 = hashlib.md5(password.encode()).hexdigest()
      result_text = '%s:%s:%s' % (passmd5 ,nonce, cnonce)
      result = hashlib.md5(result_text.encode()).hexdigest()
      sys.stderr.write('Authenticating...')
      sys.stderr.flush()
      message = '%d %s %s\n' % (AUTH, cnonce, result)
      sock2.sendto(message.encode(), remote_address)
      sock2.settimeout(10)
      try:
        data = sock2.recv(32).decode()
      except Exception:
        sys.stderr.write('FAIL\n')
        logging.error('No Answer to our Authentication')
        sock2.close()
        return 1
      if (data != "OK"):
        sys.stderr.write('FAIL\n')
        logging.error('%s', data)
        sock2.close()
        sys.exit(1)
        return 1
      sys.stderr.write('OK\n')
    else:
      logging.error('Bad Answer: %s', data)
      sock2.close()
      return 1
  sock2.close()

  logging.info('Waiting for device...')
  try:
    sock.settimeout(10)
    connection, client_address = sock.accept()
    sock.settimeout(None)
    connection.settimeout(None)
  except Exception:
    logging.error('No response from device')
    sock.close()
    return 1

  received_ok = False

  try:
    f = open(filename, "rb")
    if (PROGRESS):
      update_progress(0)
    else:
      sys.stderr.write('Uploading')
      sys.stderr.flush()
    offset = 0
    while True:
      chunk = f.read(1460)
      if not chunk: break
      offset += len(chunk)
      update_progress(offset/float(content_size))
      connection.settimeout(10)
      try:
        connection.sendall(chunk)
        if connection.recv(32).decode().find('O') >= 0:
          # connection will receive only digits or 'OK'
          received_ok = True
      except Exception:
        sys.stderr.write('\n')
        logging.error('Error Uploading')
        connection.close()
        f.close()
        sock.close()
        return 1

    sys.stderr.write('\n')
    logging.info('Waiting for result...')
    # libraries/ArduinoOTA/ArduinoOTA.cpp L311 L320
    # only sends digits or 'OK'. We must not not close
    # the connection before receiving the 'O' of 'OK'
    try:
      connection.settimeout(60)
      received_ok = False
      received_error = False
      while not (received_ok or received_error):
        reply = connection.recv(64).decode()
        # Look for either the "E" in ERROR or the "O" in OK response
        # Check for "E" first, since both strings contain "O"
        if reply.find('E') >= 0:
          sys.stderr.write('\n')
          logging.error('%s', reply)
          received_error = True
        elif reply.find('O') >= 0:
          logging.info('Result: OK')
          received_ok = True
      connection.close()
      f.close()
      sock.close()
      if received_ok:
        return 0
      return 1
    except Exception:
      logging.error('No Result!')
      connection.close()
      f.close()
      sock.close()
      return 1

  finally:
    connection.close()
    f.close()

  sock.close()
  return 1
# end serve


def parser(unparsed_args):
  parser = optparse.OptionParser(
    usage = "%prog [options]",
    description = "Transmit image over the air to the esp8266 module with OTA support."
  )

  # destination ip and port
  group = optparse.OptionGroup(parser, "Destination")
  group.add_option("-i", "--ip",
    dest = "esp_ip",
    action = "store",
    help = "ESP8266 IP Address.",
    default = False
  )
  group.add_option("-I", "--host_ip",
    dest = "host_ip",
    action = "store",
    help = "Host IP Address.",
    default = "0.0.0.0"
  )
  group.add_option("-p", "--port",
    dest = "esp_port",
    type = "int",
    help = "ESP8266 ota Port. Default 8266",
    default = 8266
  )
  group.add_option("-P", "--host_port",
    dest = "host_port",
    type = "int",
    help = "Host server ota Port. Default random 10000-60000",
    default = random.randint(10000,60000)
  )
  parser.add_option_group(group)

  # auth
  group = optparse.OptionGroup(parser, "Authentication")
  group.add_option("-a", "--auth",
    dest = "auth",
    help = "Set authentication password.",
    action = "store",
    default = ""
  )
  parser.add_option_group(group)

  # image
  group = optparse.OptionGroup(parser, "Image")
  group.add_option("-f", "--file",
    dest = "image",
    help = "Image file.",
    metavar="FILE",
    default = None
  )
  group.add_option("-s", "--spiffs",
    dest = "spiffs",
    action = "store_true",
    help = "Use this option to transmit a SPIFFS image and do not flash the module.",
    default = False
  )
  parser.add_option_group(group)

  # output group
  group = optparse.OptionGroup(parser, "Output")
  group.add_option("-d", "--debug",
    dest = "debug",
    help = "Show debug output. And override loglevel with debug.",
    action = "store_true",
    default = False
  )
  group.add_option("-r", "--progress",
    dest = "progress",
    help = "Show progress output. Does not work for ArduinoIDE",
    action = "store_true",
    default = False
  )
  parser.add_option_group(group)

  (options, args) = parser.parse_args(unparsed_args)

  return options
# end parser


def main(args):
  # get options
  options = parser(args)

  # adapt log level
  loglevel = logging.WARNING
  if (options.debug):
    loglevel = logging.DEBUG
  # end if

  # logging
  logging.basicConfig(level = loglevel, format = '%(asctime)-8s [%(levelname)s]: %(message)s', datefmt = '%H:%M:%S')

  logging.debug("Options: %s", str(options))

  # check options
  global PROGRESS
  PROGRESS = options.progress
  if (not options.esp_ip or not options.image):
    logging.critical("Not enough arguments.")

    return 1
  # end if

  command = FLASH
  if (options.spiffs):
    command = SPIFFS
  # end if

  return serve(options.esp_ip, options.host_ip, options.esp_port, options.host_port, options.auth, options.image, command)
# end main


if __name__ == '__main__':
  sys.exit(main(sys.argv))
# end if



#!/usr/bin/env python3

# Display the segment sizes used by an ELF
#
# Copyright (C) 2019 - Earle F. Philhower, III
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import print_function
import argparse
import os
import subprocess
import sys

def get_segment_hints(iram):
    hints = {}
    hints['ICACHE'] = '          - flash instruction cache'
    hints['IROM'] = '         - code in flash         (default or ICACHE_FLASH_ATTR)'
    hints['IRAM'] = '  / ' + str(iram) + ' - code in IRAM          (IRAM_ATTR, ISRs...)'
    hints['DATA'] = ')         - initialized variables (global, static) in RAM/HEAP'
    hints['RODATA'] = ') / 81920 - constants             (global, static) in RAM/HEAP'
    hints['BSS'] = ')         - zeroed variables      (global, static) in RAM/HEAP'
    return hints

def get_segment_sizes(elf, path):
    sizes = {}
    sizes['ICACHE'] = 0
    sizes['IROM'] = 0
    sizes['IRAM'] = 0
    sizes['DATA'] = 0
    sizes['RODATA'] = 0
    sizes['BSS'] = 0
    p = subprocess.Popen([path + "/xtensa-lx106-elf-size", '-A', elf], stdout=subprocess.PIPE, universal_newlines=True )
    lines = p.stdout.readlines()
    for line in lines:
        words = line.split()
        if line.startswith('.irom0.text'):
            sizes['IROM'] = sizes['IROM'] + int(words[1])
        elif line.startswith('.text'): # Gets .text and .text1
            sizes['IRAM'] = sizes['IRAM'] + int(words[1])
        elif line.startswith('.data'): # Gets .text and .text1
            sizes['DATA'] = sizes['DATA'] + int(words[1])
        elif line.startswith('.rodata'): # Gets .text and .text1
            sizes['RODATA'] = sizes['RODATA'] + int(words[1])
        elif line.startswith('.bss'): # Gets .text and .text1
            sizes['BSS'] = sizes['BSS'] + int(words[1])
    return sizes

def get_mmu_sizes(mmu, sizes):
    iram = 0x8000
    sizes['ICACHE'] = 0x8000
    lines = mmu.split(' ')
    for line in lines:
        words = line.split('=')
        if line.startswith('-DMMU_IRAM_SIZE'):
            iram = int(words[1], 16)
        elif line.startswith('-DMMU_ICACHE_SIZE'):
            sizes['ICACHE'] = int(words[1], 16)
    return [iram, sizes]

def main():
    parser = argparse.ArgumentParser(description='Report the different segment sizes of a compiled ELF file')
    parser.add_argument('-e', '--elf', action='store', required=True, help='Path to the Arduino sketch ELF')
    parser.add_argument('-p', '--path', action='store', required=True, help='Path to Xtensa toolchain binaries')
    parser.add_argument('-i', '--mmu', action='store', required=False, help='MMU build options')

    args = parser.parse_args()
    sizes = get_segment_sizes(args.elf, args.path)
    [iram, sizes] = get_mmu_sizes(args.mmu, sizes)
    hints = get_segment_hints(iram)

    sys.stderr.write("Executable segment sizes:" + os.linesep)
    for k in sizes.keys():
        sys.stderr.write("%-7s: %-5d %s %s" % (k, sizes[k], hints[k], os.linesep))

    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# This file consists of the common useful functions for eFuse
#
# Copyright (C) 2020 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

import esptool


def hexify(bitstring, separator=""):
    try:
        as_bytes = tuple(ord(b) for b in bitstring)
    except TypeError:  # python 3, items in bitstring already ints
        as_bytes = tuple(b for b in bitstring)
    return separator.join(("%02x" % b) for b in as_bytes)


def popcnt(b):
    """ Return number of "1" bits set in 'b' """
    return len([x for x in bin(b) if x == "1"])


def check_duplicate_name_in_list(name_list):
    duples_name = [name for i, name in enumerate(name_list) if name in name_list[:i]]
    if duples_name != []:
        raise esptool.FatalError("Found repeated {} in the name list".format(duples_name))



#!/usr/bin/env python
# efuse get/set utility
# https://github.com/themadinventor/esptool
#
# Copyright (C) 2016 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

import argparse
import os
import sys
from io import StringIO

import espressif.efuse.esp32 as esp32_efuse
import espressif.efuse.esp32c3 as esp32c3_efuse
import espressif.efuse.esp32s2 as esp32s2_efuse
import espressif.efuse.esp32s3beta2 as esp32s3beta2_efuse

import esptool


def get_esp(port, baud, connect_mode, chip='auto', skip_connect=False, virt=False, debug=False, virt_efuse_file=None):
    if chip not in ['auto', 'esp32', 'esp32s2', 'esp32s3beta2', 'esp32c3']:
        raise esptool.FatalError("get_esp: Unsupported chip (%s)" % chip)
    if virt:
        esp = {
            'esp32': esp32_efuse,
            'esp32s2': esp32s2_efuse,
            'esp32s3beta2': esp32s3beta2_efuse,
            'esp32c3': esp32c3_efuse,
        }.get(chip, esp32_efuse).EmulateEfuseController(virt_efuse_file, debug)
    else:
        if chip == 'auto' and not skip_connect:
            esp = esptool.ESPLoader.detect_chip(port, baud, connect_mode)
        else:
            esp = {
                'esp32': esptool.ESP32ROM,
                'esp32s2': esptool.ESP32S2ROM,
                'esp32s3beta2': esptool.ESP32S3BETA2ROM,
                'esp32c3': esptool.ESP32C3ROM,
            }.get(chip, esptool.ESP32ROM)(port if not skip_connect else StringIO(), baud)
            if not skip_connect:
                esp.connect(connect_mode)
    return esp


def get_efuses(esp, skip_connect=False, debug_mode=False, do_not_confirm=False):
    try:
        efuse = {
            'ESP32': esp32_efuse,
            'ESP32-S2': esp32s2_efuse,
            'ESP32-S3(beta2)': esp32s3beta2_efuse,
            'ESP32-C3': esp32c3_efuse,
        }[esp.CHIP_NAME]
    except KeyError:
        raise esptool.FatalError("get_efuses: Unsupported chip (%s)" % esp.CHIP_NAME)
    # dict mapping register name to its efuse object
    return (efuse.EspEfuses(esp, skip_connect, debug_mode, do_not_confirm), efuse.operations)


def main():
    init_parser = argparse.ArgumentParser(description='espefuse.py v%s - [ESP32/S2/S3BETA2/C3] efuse get/set tool' % esptool.__version__, prog='espefuse',
                                          add_help=False)

    init_parser.add_argument('--chip', '-c',
                             help='Target chip type',
                             choices=['auto', 'esp32', 'esp32s2', 'esp32s3beta2', 'esp32c3'],
                             default=os.environ.get('ESPTOOL_CHIP', 'auto'))

    init_parser.add_argument('--baud', '-b',
                             help='Serial port baud rate used when flashing/reading',
                             type=esptool.arg_auto_int,
                             default=os.environ.get('ESPTOOL_BAUD', esptool.ESPLoader.ESP_ROM_BAUD))

    init_parser.add_argument('--port', '-p',
                             help='Serial port device',
                             default=os.environ.get('ESPTOOL_PORT', esptool.ESPLoader.DEFAULT_PORT))

    init_parser.add_argument('--before',
                             help='What to do before connecting to the chip',
                             choices=['default_reset', 'no_reset', 'esp32r1', 'no_reset_no_sync'],
                             default='default_reset')

    init_parser.add_argument('--debug', "-d", help='Show debugging information (loglevel=DEBUG)', action='store_true')
    init_parser.add_argument('--virt', help='For host tests, the tool will work in the virtual mode (without connecting to a chip).', action='store_true')
    init_parser.add_argument('--path-efuse-file', help='For host tests, saves efuse memory to file.', type=str, default=None)
    init_parser.add_argument('--do-not-confirm', help='Do not pause for confirmation before permanently writing efuses. Use with caution.', action='store_true')

    args1, remaining_args = init_parser.parse_known_args()
    debug_mode = args1.debug or ("dump" in remaining_args)
    just_print_help = [True for arg in remaining_args if arg in ["--help", "-h"]] or remaining_args == []
    esp = get_esp(args1.port, args1.baud, args1.before, args1.chip, just_print_help, args1.virt, args1.debug, args1.path_efuse_file)
    efuses, efuse_operations = get_efuses(esp, just_print_help, debug_mode, args1.do_not_confirm)

    parser = argparse.ArgumentParser(parents=[init_parser])
    subparsers = parser.add_subparsers(dest='operation', help='Run espefuse.py {command} -h for additional help')

    efuse_operations.add_commands(subparsers, efuses)

    args = parser.parse_args(remaining_args)
    print('espefuse.py v%s' % esptool.__version__)
    if args.operation is None:
        parser.print_help()
        parser.exit(1)
    operation_func = vars(efuse_operations)[args.operation]

    # each 'operation' is a module-level function of the same name
    operation_func(esp, efuses, args)


def _main():
    try:
        main()
    except esptool.FatalError as e:
        print('\nA fatal error occurred: %s' % e)
        sys.exit(2)


if __name__ == '__main__':
    _main()



#!/usr/bin/env python
# This file describes eFuses fields and registers for ESP32 chip
#
# Copyright (C) 2020 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

from collections import namedtuple


class EfuseRegistersBase(object):
    # Coding Scheme values
    CODING_SCHEME_NONE          = 0
    CODING_SCHEME_34            = 1
    CODING_SCHEME_REPEAT        = 2
    CODING_SCHEME_NONE_RECOVERY = 3
    CODING_SCHEME_RS            = 4

    EFUSE_BURN_TIMEOUT = 0.250  # seconds


class EfuseBlocksBase(object):

    BLOCKS = None
    NamedtupleBlock = namedtuple('Block', 'name alias id rd_addr wr_addr write_disable_bit read_disable_bit len key_purpose')

    @staticmethod
    def get(tuple_block):
        return EfuseBlocksBase.NamedtupleBlock._make(tuple_block)

    def get_blocks_for_keys(self):
        list_of_names = []
        for block in self.BLOCKS:
            blk = self.get(block)
            if blk.id > 0:
                if blk.name:
                    list_of_names.append(blk.name)
                if blk.alias:
                    for alias in blk.alias:
                        list_of_names.append(alias)
        return list_of_names


class EfuseFieldsBase(object):

    NamedtupleField = namedtuple('Efuse', 'name category block word pos type write_disable_bit read_disable_bit class_type description dictionary')

    @staticmethod
    def get(tuple_field):
        return EfuseFieldsBase.NamedtupleField._make(tuple_field)



#!/usr/bin/env python3

# Generate an Arduino compatible BIN file from bootloader and sketch ELF
# Replaces esptool-ck.exe and emulates its behavior.
#
# Copyright (C) 2019 - Earle F. Philhower, III
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

from __future__ import print_function
import argparse
import re
import os
import subprocess
import sys
import tempfile

fmodeb = { 'dout': 3, 'dio': 2, 'qout': 1, 'qio': 0 }
ffreqb = { '40': 0, '26': 1, '20': 2, '80': 15 }
fsizeb = { '512K': 0, '256K': 1, '1M': 2, '2M': 3, '4M': 4, '8M': 8, '16M': 9 }

crcsize_offset = 4096 + 16
crcval_offset =  4096 + 16 + 4

def get_elf_entry(elf, path):
    p = subprocess.Popen([path + "/xtensa-lx106-elf-readelf", '-h', elf], stdout=subprocess.PIPE, universal_newlines=True )
    lines = p.stdout.readlines()
    for line in lines:
        if 'Entry point address' in line:
            words = re.split('\s+', line)
            entry_point = words[-2]
            return int(entry_point, 16)
    raise Exception('Unable to find entry point in file "' + elf + '"')

def get_segment_size_addr(elf, segment, path):
    p = subprocess.Popen([path + '/xtensa-lx106-elf-objdump', '-h', '-j', segment,  elf], stdout=subprocess.PIPE, universal_newlines=True )
    lines = p.stdout.readlines()
    for line in lines:
        if segment in line:
            words = re.split('\s+', line)
            size = int(words[3], 16)
            addr = int(words[4], 16)
            return [ size, addr ]
    raise Exception('Unable to find size and start point in file "' + elf + '" for "' + segment + '"')

def read_segment(elf, segment, path):
    fd, tmpfile = tempfile.mkstemp()
    os.close(fd)
    subprocess.check_call([path + "/xtensa-lx106-elf-objcopy", '-O', 'binary', '--only-section=' + segment, elf, tmpfile], stdout=subprocess.PIPE)
    with open(tmpfile, "rb") as f:
        raw = f.read()
    os.remove(tmpfile)

    return raw

def write_bin(out, args, elf, segments, to_addr):
    entry = int(get_elf_entry( elf, args.path ))
    header = [ 0xe9, len(segments), fmodeb[args.flash_mode], ffreqb[args.flash_freq] + 16 * fsizeb[args.flash_size],
               entry & 255, (entry>>8) & 255, (entry>>16) & 255, (entry>>24) & 255 ]
    out.write(bytearray(header))
    total_size = 8
    checksum = 0xef
    for segment in segments:
        [size, addr] = get_segment_size_addr(elf, segment, args.path)
        seghdr = [ addr & 255, (addr>>8) & 255, (addr>>16) & 255, (addr>>24) & 255,
                   size & 255, (size>>8) & 255, (size>>16) & 255, (size>>24) & 255]
        out.write(bytearray(seghdr));
        total_size += 8;
        raw = read_segment(elf, segment, args.path)
        if len(raw) != size:
            raise Exception('Segment size doesn\'t match read data for "' + segment + '" in "' + elf + '"')
        out.write(raw)
        total_size += len(raw)
        try:
            for data in raw:
                checksum = checksum ^ ord(data)
        except Exception:
            for data in raw:
                checksum = checksum ^ data
    total_size += 1
    while total_size & 15:
        total_size += 1
        out.write(bytearray([0]))
    out.write(bytearray([checksum]))
    if to_addr != 0:
        if total_size + 8 > to_addr:
            raise Exception('Bin image of ' + elf + ' is too big, actual size ' + str(total_size  + 8) + ', target size ' + str(to_addr) + '.')
        while total_size < to_addr:
            out.write(bytearray([0xaa]))
            total_size += 1

def crc8266(ldata):
    "Return the CRC of ldata using same algorithm as eboot"
    crc = 0xffffffff
    idx = 0
    while idx < len(ldata):
        byte = int(ldata[idx])
        idx = idx + 1
        for i in [0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01]:
            bit = crc & 0x80000000
            if (byte & i) != 0:
                if bit == 0:
                    bit = 1
                else:
                    bit = 0
            crc = int(crc << 1) & 0xffffffff
            if bit != 0:
                crc = int(crc ^ 0x04c11db7)
    return crc

def store_word(raw, offset, val):
    "Place a 4-byte word in 8266-dependent order in the raw image"
    raw[offset] = val & 255
    raw[offset + 1] = (val >> 8) & 255
    raw[offset + 2] = (val >> 16) & 255
    raw[offset + 3] = (val >> 24) & 255
    return raw

def add_crc(out):
    with open(out, "rb") as binfile:
        raw = bytearray(binfile.read())

    # Zero out the spots we're going to overwrite to be idempotent
    raw = store_word(raw, crcsize_offset, 0)
    raw = store_word(raw, crcval_offset, 0)
    crc = crc8266(raw)
    raw = store_word(raw, crcsize_offset, len(raw))
    raw = store_word(raw, crcval_offset, int(crc))

    with open(out, "wb") as binfile:
        binfile.write(raw)

def gzip_bin(mode, out):
    import gzip

    firmware_path = out
    gzip_path = firmware_path + '.gz'
    orig_path = firmware_path + '.orig'
    if os.path.exists(gzip_path):
        os.remove(gzip_path)
    print('GZipping firmware ' + firmware_path)
    with open(firmware_path, 'rb') as firmware_file, \
            gzip.open(gzip_path, 'wb') as dest:
        data = firmware_file.read()
        dest.write(data)
    orig_size = os.stat(firmware_path).st_size
    gzip_size = os.stat(gzip_path).st_size
    print("New FW size {:d} bytes vs old {:d} bytes".format(
        gzip_size, orig_size))

    if mode == "PIO":
        if os.path.exists(orig_path):
            os.remove(orig_path)
        print('Moving original firmware to ' + orig_path)
        os.rename(firmware_path, orig_path)
        os.rename(gzip_path, firmware_path)

def main():
    parser = argparse.ArgumentParser(description='Create a BIN file from eboot.elf and Arduino sketch.elf for upload by esptool.py')
    parser.add_argument('-e', '--eboot', action='store', required=True, help='Path to the Arduino eboot.elf bootloader')
    parser.add_argument('-a', '--app', action='store', required=True, help='Path to the Arduino sketch ELF')
    parser.add_argument('-m', '--flash_mode', action='store', required=True, choices=['dout', 'dio', 'qout', 'qio'], help='SPI flash mode')
    parser.add_argument('-f', '--flash_freq', action='store', required=True, choices=['20', '26', '40', '80'], help='SPI flash speed')
    parser.add_argument('-s', '--flash_size', action='store', required=True, choices=['256K', '512K', '1M', '2M', '4M', '8M', '16M'], help='SPI flash size')
    parser.add_argument('-o', '--out', action='store', required=True, help='Output BIN filename')
    parser.add_argument('-p', '--path', action='store', required=True, help='Path to Xtensa toolchain binaries')
    parser.add_argument('-g', '--gzip', choices=['PIO', 'Arduino'], help='PIO - generate gzipped BIN file, Arduino - generate BIN and BIN.gz')

    args = parser.parse_args()

    print('Creating BIN file "{out}" using "{eboot}" and "{app}"'.format(
        out=args.out, eboot=args.eboot, app=args.app))

    with open(args.out, "wb") as out:
        def wrapper(**kwargs):
            write_bin(out=out, args=args, **kwargs)

        wrapper(
            elf=args.eboot,
            segments=[".text", ".rodata"],
            to_addr=4096
        )

        wrapper(
            elf=args.app,
            segments=[".irom0.text", ".text", ".text1", ".data", ".rodata"],
            to_addr=0
        )

    # Because the CRC includes both eboot and app, can only calculate it after the entire BIN generated
    add_crc(args.out)

    if args.gzip:
        gzip_bin(args.gzip, args.out)

    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# This file describes eFuses fields and registers for ESP32-S3(beta2) chip
#
# Copyright (C) 2020 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

from ..mem_definition_base import EfuseBlocksBase, EfuseFieldsBase, EfuseRegistersBase


class EfuseDefineRegisters(EfuseRegistersBase):

    EFUSE_ADDR_MASK = 0x00000FFF
    EFUSE_MEM_SIZE = (0x01FC + 4)

    # EFUSE registers & command/conf values
    DR_REG_EFUSE_BASE       = 0x6001A000
    EFUSE_PGM_DATA0_REG     = DR_REG_EFUSE_BASE
    EFUSE_CHECK_VALUE0_REG  = DR_REG_EFUSE_BASE + 0x020
    EFUSE_CLK_REG           = DR_REG_EFUSE_BASE + 0x1C8
    EFUSE_CONF_REG          = DR_REG_EFUSE_BASE + 0x1CC
    EFUSE_STATUS_REG        = DR_REG_EFUSE_BASE + 0x1D0
    EFUSE_CMD_REG           = DR_REG_EFUSE_BASE + 0x1D4
    EFUSE_RD_RS_ERR0_REG    = DR_REG_EFUSE_BASE + 0x1C0
    EFUSE_RD_RS_ERR1_REG    = DR_REG_EFUSE_BASE + 0x1C4
    EFUSE_WRITE_OP_CODE     = 0x5A5A
    EFUSE_READ_OP_CODE      = 0x5AA5
    EFUSE_PGM_CMD_MASK      = 0x3
    EFUSE_PGM_CMD           = 0x2
    EFUSE_READ_CMD          = 0x1

    BLOCK_ERRORS = [
        # error reg,            err_num,    fail_bit
        (None,                  None,       None),  # BLOCK0
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 0,     3),     # MAC_SPI_8M_0
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 4,     7),     # BLOCK_SYS_DATA
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 8,     11),    # BLOCK_USR_DATA
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 12,    15),    # BLOCK_KEY0
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 16,    19),    # BLOCK_KEY1
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 20,    23),    # BLOCK_KEY2
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 24,    27),    # BLOCK_KEY3
        (EFUSE_RD_RS_ERR0_REG,  0x7 << 28,    31),    # BLOCK_KEY4
        (EFUSE_RD_RS_ERR1_REG,  0x7 << 0,     3),     # BLOCK_KEY5
        (EFUSE_RD_RS_ERR1_REG,  0x7 << 4,     7),     # BLOCK_KEY6
    ]

    EFUSE_WR_TIM_CONF2_REG = DR_REG_EFUSE_BASE + 0x1F8
    EFUSE_PWR_OFF_NUM_S = 0
    EFUSE_PWR_OFF_NUM_M = 0xFFFF << EFUSE_PWR_OFF_NUM_S


class EfuseDefineBlocks(EfuseBlocksBase):

    __base_rd_regs = EfuseDefineRegisters.DR_REG_EFUSE_BASE
    __base_wr_regs = EfuseDefineRegisters.EFUSE_PGM_DATA0_REG
    # List of efuse blocks
    BLOCKS = [
        # Name,             Alias,   Index,  Read address,           Write address,   Write protect bit, Read protect bit, Len, key_purpose
        ("BLOCK0",          [],          0,  __base_rd_regs + 0x02C, __base_wr_regs, None, None, 6, None),
        ("MAC_SPI_8M_0",    ["BLOCK1"],  1,  __base_rd_regs + 0x044, __base_wr_regs, 20,   None, 6, None),
        ("BLOCK_SYS_DATA",  ["BLOCK2"],  2,  __base_rd_regs + 0x05C, __base_wr_regs, 21,   None, 8, None),
        ("BLOCK_USR_DATA",  ["BLOCK3"],  3,  __base_rd_regs + 0x07C, __base_wr_regs, 22,   None, 8, None),
        ("BLOCK_KEY0",      ["BLOCK4"],  4,  __base_rd_regs + 0x09C, __base_wr_regs, 23,   0,    8, "KEY_PURPOSE_0"),
        ("BLOCK_KEY1",      ["BLOCK5"],  5,  __base_rd_regs + 0x0BC, __base_wr_regs, 24,   1,    8, "KEY_PURPOSE_1"),
        ("BLOCK_KEY2",      ["BLOCK6"],  6,  __base_rd_regs + 0x0DC, __base_wr_regs, 25,   2,    8, "KEY_PURPOSE_2"),
        ("BLOCK_KEY3",      ["BLOCK7"],  7,  __base_rd_regs + 0x0FC, __base_wr_regs, 26,   3,    8, "KEY_PURPOSE_3"),
        ("BLOCK_KEY4",      ["BLOCK8"],  8,  __base_rd_regs + 0x11C, __base_wr_regs, 27,   4,    8, "KEY_PURPOSE_4"),
        ("BLOCK_KEY5",      ["BLOCK9"],  9,  __base_rd_regs + 0x13C, __base_wr_regs, 28,   5,    8, "KEY_PURPOSE_5"),
        ("BLOCK_SYS_DATA2", ["BLOCK10"], 10, __base_rd_regs + 0x15C, __base_wr_regs, 29,   6,    8, None),
    ]

    def get_burn_block_data_names(self):
        list_of_names = []
        for block in self.BLOCKS:
            blk = self.get(block)
            if blk.name:
                list_of_names.append(blk.name)
            if blk.alias:
                for alias in blk.alias:
                    list_of_names.append(alias)
        return list_of_names


class EfuseDefineFields(EfuseFieldsBase):

    # List of efuse fields from TRM the chapter eFuse Controller.
    EFUSES = [
        #
        # Table 51: Parameters in BLOCK0
        # Name                           Category Block Word Pos Type:len   WR_DIS RD_DIS Class        Description                Dictionary
        ("WR_DIS",                       "efuse",    0,  0,  0,  "uint:32",  None, None, None,         "Disables programming of individual eFuses", None),
        ("RD_DIS",                       "efuse",    0,  1,  0,  "uint:7",   0,    None, None,         "Disables software reading from BLOCK4-10", None),
        ("DIS_RTC_RAM_BOOT",             "config",   0,  1,  7,  "bool",     1,    None, None,         "Disables boot from RTC RAM", None),
        ("DIS_ICACHE",                   "config",   0,  1,  8,  "bool",     2,    None, None,         "Disables ICache", None),
        ("DIS_DCACHE",                   "config",   0,  1,  9,  "bool",     2,    None, None,         "Disables DCache", None),
        ("DIS_DOWNLOAD_ICACHE",          "config",   0,  1,  10, "bool",     2,    None, None,         "Disables Icache when SoC is in Download mode", None),
        ("DIS_DOWNLOAD_DCACHE",          "config",   0,  1,  11, "bool",     2,    None, None,         "Disables Dcache when SoC is in Download mode", None),
        ("DIS_FORCE_DOWNLOAD",           "config",   0,  1,  12, "bool",     2,    None, None,         "Disables forcing chip into Download mode", None),
        ("DIS_USB",                  "usb config",   0,  1,  13, "bool",     2,    None, None,         "Disables the USB OTG hardware", None),
        ("DIS_CAN",                      "config",   0,  1,  14, "bool",     2,    None, None,         "Disables the TWAI Controller hardware", None),
        ("DIS_APP_CPU",                  "config",   0,  1,  15, "bool",     2,    None, None,         "Disables APP CPU", None),
        ("SOFT_DIS_JTAG",                "security", 0,  1,  16, "uint:3",   31,   None, None,         "Software disables JTAG by programming "
                                                                                                       "odd number of 1 bit(s). "
                                                                                                       "JTAG can be re-enabled via HMAC peripheral",
                                                                                                       None),
        ("HARD_DIS_JTAG",                "security", 0,  1,  19, "bool",     2,    None, None,         "Hardware disables JTAG permanently", None),

        ("DIS_DOWNLOAD_MANUAL_ENCRYPT",  "security", 0,  1,  20, "bool",     2,    None, None,         "Disables flash encryption when in download boot modes",
                                                                                                       None),
        ("USB_EXCHG_PINS",           "usb config",   0,  1,  25, "bool",     30,   None, None,         "Exchanges USB D+ and D- pins", None),
        ("EXT_PHY_ENABLE",           "usb config",   0,  1,  26, "bool",     30,   None, None,         "Enables external USB PHY", None),
        ("BTLC_GPIO_ENABLE",         "usb config",   0,  1,  27, "uint:2",   30,   None, None,         "Enables BTLC GPIO", None),
        ("VDD_SPI_XPD",          "VDD_SPI config",   0,  2,  4,  "bool",     3,    None, None,         "The VDD_SPI regulator is powered on", None),
        ("VDD_SPI_TIEH",         "VDD_SPI config",   0,  2,  5,  "bool",     3,    None, None,         "The VDD_SPI power supply voltage at reset",
         {0: "Connect to 1.8V LDO",
          1: "Connect to VDD_RTC_IO"}),
        ("VDD_SPI_FORCE",        "VDD_SPI config",   0,  2,  6,  "bool",     3,    None, None,         "Force using VDD_SPI_XPD and VDD_SPI_TIEH "
                                                                                                       "to configure VDD_SPI LDO", None),
        ("WDT_DELAY_SEL",            "WDT config",   0,  2,  16, "uint:2",   3,    None, None,         "Selects RTC WDT timeout threshold at startup", None),
        ("SPI_BOOT_CRYPT_CNT",         "security",   0,  2,  18, "uint:3",   4,    None, None,         "Enables encryption and decryption, when an SPI boot "
                                                                                                       "mode is set. Enabled when 1 or 3 bits are set,"
                                                                                                       "disabled otherwise",
         {0: "Disable",
          1: "Enable",
          3: "Disable",
          7: "Enable"}),
        ("SECURE_BOOT_KEY_REVOKE0",      "security", 0,  2, 21,  "bool",     5,    None, None,         "Revokes use of secure boot key digest 0", None),
        ("SECURE_BOOT_KEY_REVOKE1",      "security", 0,  2, 22,  "bool",     6,    None, None,         "Revokes use of secure boot key digest 1", None),
        ("SECURE_BOOT_KEY_REVOKE2",      "security", 0,  2, 23,  "bool",     7,    None, None,         "Revokes use of secure boot key digest 2", None),
        ("KEY_PURPOSE_0",                "security", 0,  2, 24,  "uint:4",   8,    None, "keypurpose", "KEY0 purpose", None),
        ("KEY_PURPOSE_1",                "security", 0,  2, 28,  "uint:4",   9,    None, "keypurpose", "KEY1 purpose", None),
        ("KEY_PURPOSE_2",                "security", 0,  3, 0,   "uint:4",   10,   None, "keypurpose", "KEY2 purpose", None),
        ("KEY_PURPOSE_3",                "security", 0,  3, 4,   "uint:4",   11,   None, "keypurpose", "KEY3 purpose", None),
        ("KEY_PURPOSE_4",                "security", 0,  3, 8,   "uint:4",   12,   None, "keypurpose", "KEY4 purpose", None),
        ("KEY_PURPOSE_5",                "security", 0,  3, 12,  "uint:4",   13,   None, "keypurpose", "KEY5 purpose", None),
        ("SECURE_BOOT_EN",               "security", 0,  3, 20,  "bool",     15,   None, None,         "Enables secure boot", None),
        ("SECURE_BOOT_AGGRESSIVE_REVOKE", "security", 0, 3, 21,  "bool",     16,   None, None,         "Enables aggressive secure boot key revocation mode",
                                                                                                       None),
        ("FLASH_TPUW",                   "config",   0,  3, 28,  "uint:4",   18,   None, None,         "Configures flash startup delay after SoC power-up, "
                                                                                                       "unit is (ms/2). When the value is 15, delay is 7.5 ms",
                                                                                                       None),
        ("DIS_DOWNLOAD_MODE",            "security", 0,  4, 0,   "bool",     18,   None, None,         "Disables all Download boot modes", None),
        ("DIS_LEGACY_SPI_BOOT",          "config",   0,  4, 1,   "bool",     18,   None, None,         "Disables Legacy SPI boot mode", None),
        ("UART_PRINT_CHANNEL",           "config",   0,  4, 2,   "bool",     18,   None, None,         "Selects the default UART for printing boot msg",
         {0: "UART0",
          1: "UART1"}),
        ("FLASH_ECC_MODE",               "config",   0,  4, 3,   "bool",     18,   None, None,         "Configures the ECC mode for SPI flash",
         {0: "16-byte to 18-byte mode",
          1: "16-byte to 17-byte mode"}),
        ("DIS_USB_DOWNLOAD_MODE",        "config",   0,  4, 4,   "bool",     18,   None, None,         "Disables USB OTG download feature in "
                                                                                                       "UART download boot mode", None),
        ("ENABLE_SECURITY_DOWNLOAD",    "security",  0,  4, 5,   "bool",     18,   None, None,         "Enables secure UART download mode "
                                                                                                       "(read/write flash only)", None),
        ("UART_PRINT_CONTROL",           "config",   0,  4, 6,   "uint:2",   18,   None, None,         "Sets the default UART boot message output mode",
         {0: "Enabled",
          1: "Enable when GPIO 46 is low at reset",
          2: "Enable when GPIO 46 is high at rest",
          3: "Disabled"}),
        ("PIN_POWER_SELECTION",  "VDD_SPI config",   0,  4, 8,   "bool",     18,   None, None,         "Sets default power supply for GPIO33..37",
         {0: "VDD3P3_CPU",
          1: "VDD_SPI"}),
        ("FLASH_TYPE",                   "config",   0,  4, 9,   "bool",     18,   None, None,         "Selects SPI flash type",
         {0: "4 data lines",
          1: "8 data lines"}),
        ("FLASH_PAGE_SIZE",              "config",   0,  4, 10,  "uint:2",   18,   None, None,         "Sets the size of flash page", None),
        ("FLASH_ECC_EN",                 "config",   0,  4, 12,  "bool",     18,   None, None,         "Enables ECC in Flash boot mode", None),
        ("FORCE_SEND_RESUME",            "config",   0,  4, 13,  "bool",     18,   None, None,         "Forces ROM code to send an SPI flash resume command "
                                                                                                       "during SPI boot", None),
        ("SECURE_VERSION",             "identity",   0,  4, 14,  "uint:16",  18,   None, "bitcount",   "Secure version (used by ESP-IDF anti-rollback feature)",
                                                                                                       None),
        #
        # Table 53: Parameters in BLOCK1-10
        # Name                          Category  Block Word Pos  Type:len WR_DIS RD_DIS Class         Description                Dictionary
        ("MAC",                        "identity",   1,  0, 0,   "bytes:6",  20,   None, "mac",        "Factory MAC Address", None),
        ("SPI_PAD_CONFIG_CLK",   "spi_pad_config",   1,  1, 16,  "uint:6",   20,   None, None,         "SPI CLK pad", None),
        ("SPI_PAD_CONFIG_Q",     "spi_pad_config",   1,  1, 22,  "uint:6",   20,   None, None,         "SPI Q (D1) pad", None),
        ("SPI_PAD_CONFIG_D",     "spi_pad_config",   1,  1, 28,  "uint:6",   20,   None, None,         "SPI D (D0) pad", None),
        ("SPI_PAD_CONFIG_CS",    "spi_pad_config",   1,  2, 2,   "uint:6",   20,   None, None,         "SPI CS pad", None),
        ("SPI_PAD_CONFIG_HD",    "spi_pad_config",   1,  2, 8,   "uint:6",   20,   None, None,         "SPI HD (D3) pad", None),
        ("SPI_PAD_CONFIG_WP",    "spi_pad_config",   1,  2, 14,  "uint:6",   20,   None, None,         "SPI WP (D2) pad", None),
        ("SPI_PAD_CONFIG_DQS",   "spi_pad_config",   1,  2, 20,  "uint:6",   20,   None, None,         "SPI DQS pad", None),
        ("SPI_PAD_CONFIG_D4",    "spi_pad_config",   1,  2, 26,  "uint:6",   20,   None, None,         "SPI D4 pad", None),
        ("SPI_PAD_CONFIG_D5",    "spi_pad_config",   1,  3, 0,   "uint:6",   20,   None, None,         "SPI D5 pad", None),
        ("SPI_PAD_CONFIG_D6",    "spi_pad_config",   1,  3, 6,   "uint:6",   20,   None, None,         "SPI D6 pad", None),
        ("SPI_PAD_CONFIG_D7",    "spi_pad_config",   1,  3, 12,  "uint:6",   20,   None, None,         "SPI D7 pad", None),
        ("WAFER_VERSION",              "identity",   1,  3, 18,  "uint:3",   20,   None, None,         "WAFER version",
         {0: "A"}),
        ("PKG_VERSION",                "identity",   1,  3, 21,  "uint:4",   20,   None, None,         "??? Package version",
         {0: "ESP32-S3"}),
        ("BLOCK1_VERSION",             "identity",   1,  3, 25,  "uint:3",   20,   None, None,         "??? BLOCK1 efuse version", None),
        ('OPTIONAL_UNIQUE_ID',         "identity",   2,  0, 0,   "bytes:16", 21,   None, "keyblock",   "??? Optional unique 128-bit ID", None),
        ('BLOCK2_VERSION',             "identity",   2,  4, 4,   "uint:3",   21,   None, None,         "??? Version of BLOCK2",
         {0: "No calibration",
          1: "With calibration"}),
    ]

    KEYBLOCKS = [
        # Name                      Category      Block Word Pos Type:len  WR_DIS RD_DIS Class         Description                Dictionary
        ('BLOCK_USR_DATA',               "config",   3,  0, 0,   "bytes:32", 22,   None, None,         "User data", None),
        ('BLOCK_KEY0',                   "security", 4,  0, 0,   "bytes:32", 23,   0,    "keyblock",   "Encryption key0 or user data", None),
        ('BLOCK_KEY1',                   "security", 5,  0, 0,   "bytes:32", 24,   1,    "keyblock",   "Encryption key1 or user data", None),
        ('BLOCK_KEY2',                   "security", 6,  0, 0,   "bytes:32", 25,   2,    "keyblock",   "Encryption key2 or user data", None),
        ('BLOCK_KEY3',                   "security", 7,  0, 0,   "bytes:32", 26,   3,    "keyblock",   "Encryption key3 or user data", None),
        ('BLOCK_KEY4',                   "security", 8,  0, 0,   "bytes:32", 27,   4,    "keyblock",   "Encryption key4 or user data", None),
        ('BLOCK_KEY5',                   "security", 9,  0, 0,   "bytes:32", 28,   5,    "keyblock",   "Encryption key5 or user data", None),
        ('BLOCK_SYS_DATA2',              "security", 10, 0, 0,   "bytes:32", 29,   6,    None,         "System data (part 2)", None),
    ]

    # if BLOCK2_VERSION is 1, these efuse fields are in BLOCK2
    BLOCK2_CALIBRATION_EFUSES = [
        # Name                      Category      Block Word Pos Type:len  WR_DIS RD_DIS Class         Description                Dictionary
        ('TEMP_SENSOR_CAL',         "calibration",   2,  4, 7,   "uint:9",   21,   None, "t_sensor",   "??? Temperature calibration", None),
        ('ADC1_MODE0_D2',           "calibration",   2,  4, 16,  "uint:8",   21,   None, "adc_tp",     "??? ADC1 calibration 1", None),
        ('ADC1_MODE1_D2',           "calibration",   2,  4, 24,  "uint:8",   21,   None, "adc_tp",     "??? ADC1 calibration 2", None),
        ('ADC1_MODE2_D2',           "calibration",   2,  5, 0,   "uint:8",   21,   None, "adc_tp",     "??? ADC1 calibration 3", None),
        ('ADC1_MODE3_D2',           "calibration",   2,  5, 8,   "uint:8",   21,   None, "adc_tp",     "??? ADC1 calibration 4", None),
        ('ADC2_MODE0_D2',           "calibration",   2,  5, 16,  "uint:8",   21,   None, "adc_tp",     "??? ADC2 calibration 5", None),
        ('ADC2_MODE1_D2',           "calibration",   2,  5, 24,  "uint:8",   21,   None, "adc_tp",     "??? ADC2 calibration 6", None),
        ('ADC2_MODE2_D2',           "calibration",   2,  6, 0,   "uint:8",   21,   None, "adc_tp",     "??? ADC2 calibration 7", None),
        ('ADC2_MODE3_D2',           "calibration",   2,  6, 8,   "uint:8",   21,   None, "adc_tp",     "??? ADC2 calibration 8", None),
        ('ADC1_MODE0_D1',           "calibration",   2,  6, 16,  "uint:6",   21,   None, "adc_tp",     "??? ADC1 calibration 9", None),
        ('ADC1_MODE1_D1',           "calibration",   2,  6, 22,  "uint:6",   21,   None, "adc_tp",     "??? ADC1 calibration 10", None),
        ('ADC1_MODE2_D1',           "calibration",   2,  6, 28,  "uint:6",   21,   None, "adc_tp",     "??? ADC1 calibration 11", None),
        ('ADC1_MODE3_D1',           "calibration",   2,  7, 2,   "uint:6",   21,   None, "adc_tp",     "??? ADC1 calibration 12", None),
        ('ADC2_MODE0_D1',           "calibration",   2,  7, 8,   "uint:6",   21,   None, "adc_tp",     "??? ADC2 calibration 13", None),
        ('ADC2_MODE1_D1',           "calibration",   2,  7, 14,  "uint:6",   21,   None, "adc_tp",     "??? ADC2 calibration 14", None),
        ('ADC2_MODE2_D1',           "calibration",   2,  7, 20,  "uint:6",   21,   None, "adc_tp",     "??? ADC2 calibration 15", None),
        ('ADC2_MODE3_D1',           "calibration",   2,  7, 26,  "uint:6",   21,   None, "adc_tp",     "??? ADC2 calibration 16", None),
    ]



#!/usr/bin/env python3

# Platform-independent `mkdir`

import argparse
import pathlib
import sys

def main():
    parser = argparse.ArgumentParser(description='Platform-independent `mkdir`')
    parser.add_argument('-p', '--parents', action='store_true', required=False, help='no error if existing, make parent directories as needed')
    parser.add_argument('dir', action='store', nargs='+')
    ns = parser.parse_args()
    for p in ns.dir:
        try:
            pathlib.Path(p).mkdir(parents=ns.parents)
        except FileExistsError:
            pass
    return 0

if __name__ == '__main__':
    sys.exit(main())



from mock_decorators import setup, teardown, setenv, request_env

@setup('environment variables can be set and read from python')
def setup_envtest(e):
    setenv(e, 'VAR_FROM_PYTHON', '42')


@teardown('environment variables can be set and read from python')
def teardown_envtest(e):
    env_value = request_env(e, 'VAR_FROM_TEST')
    assert(env_value == '24')



#!/usr/bin/env python3

# Platform-independent single-file `cp`

import argparse
import shutil
import sys

def main():
    parser = argparse.ArgumentParser(description='Platform-independent single-file `cp`')
    parser.add_argument('src', action='store')
    parser.add_argument('dst', action='store')
    ns = parser.parse_args()
    shutil.copyfile(ns.src, ns.dst)
    return 0

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
#
# HOST_TEST for espefuse.py [support esp32, esp32s2, esp32s3beta2, esp32c3]
#
# How to use it:
#
# 1. Run as HOST_TEST (without a physical connection to a chip):
#    - `python test_espefuse_host.py esp32`
#    - `python test_espefuse_host.py esp32s2`
#
# 2. Run as TEST on FPGA (connection to FPGA with an image ESP32 or ESP32-S2):
#    required two COM ports
#    - `python test_espefuse_host.py esp32   /dev/ttyUSB0 /dev/ttyUSB1`
#    - `python test_espefuse_host.py esp32s2 /dev/ttyUSB0 /dev/ttyUSB1`
#
# where  - ttyUSB0 - a port for espefuse.py operation
#        - ttyUSB1 - a port to clear efuses (connect RTS or DTR ->- J14 pin 39)
#
# Note: For FPGA with ESP32 image, need to add a line into esptool.py
#            time.sleep(7)  # FPGA delay
#       after this line:
#            self._setDTR(False)  # IO0=HIGH, done
#       because the long delay (~6 seconds) after resetting the FPGA.
#       For FPGA with ESP32-S2 image, it is not necessary
from __future__ import division, print_function

import os
import subprocess
import sys
import tempfile
import time
import unittest

from bitstring import BitString

import serial

TEST_DIR = os.path.abspath(os.path.dirname(__file__))
os.chdir(TEST_DIR)
sys.path.insert(0, os.path.join(TEST_DIR, ".."))

support_list_chips = ["esp32", "esp32s2", "esp32s3beta2", "esp32c3"]

try:
    chip_target = sys.argv[1]
except IndexError:
    chip_target = "esp32"

global reset_port
reset_port = None
global espefuse_port
espefuse_port = None


class EfuseTestCase(unittest.TestCase):

    def setUp(self):
        if reset_port is None:
            self.efuse_file = tempfile.NamedTemporaryFile()
            self.base_cmd = "python ../espefuse.py --chip {} --virt --path-efuse-file {} -d ".format(chip_target, self.efuse_file.name)
        else:
            self.base_cmd = "python ../espefuse.py --chip {} -p {} -d ".format(chip_target, espefuse_port)
            self.reset_efuses()

    def tearDown(self):
        if reset_port is None:
            self.efuse_file.close()

    def reset_efuses(self):
        # reset and zero efuses
        reset_port.dtr = False
        reset_port.rts = False
        time.sleep(0.05)
        reset_port.dtr = True
        reset_port.rts = True
        time.sleep(0.05)
        reset_port.dtr = False
        reset_port.rts = False

    def get_esptool(self):
        if espefuse_port is not None:
            import esptool
            esp = esptool.ESPLoader.detect_chip(port=espefuse_port)
            del esptool
        else:
            if chip_target == "esp32":
                import espressif.efuse.esp32 as efuse
            elif chip_target == "esp32s2":
                import espressif.efuse.esp32s2 as efuse
            elif chip_target == "esp32s3beta2":
                import espressif.efuse.esp32s3beta2 as efuse
            elif chip_target == "esp32c3":
                import espressif.efuse.esp32c3 as efuse
            else:
                efuse = None
            esp = efuse.EmulateEfuseController(self.efuse_file.name)
            del efuse
        return esp

    def _set_34_coding_scheme(self):
        self.espefuse_py('burn_efuse CODING_SCHEME 1')

    def check_data_block_in_log(self, log, file_path, repeat=1, reverse_order=False, offset=0):
        with open(file_path, 'rb') as f:
            data = BitString('0x00') * offset + BitString(f)
            blk = data.readlist("%d*uint:8" % (data.len // 8))
            blk = blk[::-1] if reverse_order else blk
            hex_blk = " ".join("{:02x}".format(num) for num in blk)
            self.assertEqual(repeat, log.count(hex_blk))

    def espefuse_not_virt_py(self, cmd, check_msg=None, ret_code=0):
        full_cmd = ' '.join(('python ../espefuse.py', cmd))
        return self._run_command(full_cmd, check_msg, ret_code)

    def espefuse_py(self, cmd, do_not_confirm=True, check_msg=None, ret_code=0):
        full_cmd = ' '.join((self.base_cmd, '--do-not-confirm' if do_not_confirm else '', cmd))
        return self._run_command(full_cmd, check_msg, ret_code)

    def _run_command(self, cmd, check_msg, ret_code):
        try:
            p = subprocess.Popen(cmd.split(), shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, universal_newlines=True)
            returncode = p.wait()
            output, _ = p.communicate()
            if check_msg:
                self.assertIn(check_msg, output)
            if returncode:
                print(output)
            self.assertEqual(ret_code, returncode)
            return output
        except subprocess.CalledProcessError as error:
            print(error)
            raise


class TestReadCommands(EfuseTestCase):

    def test_help(self):
        self.espefuse_not_virt_py("--help")
        self.espefuse_not_virt_py("--chip %s --help" % (chip_target))

    def test_dump(self):
        self.espefuse_py("dump -h")
        self.espefuse_py("dump")

    def test_summary(self):
        self.espefuse_py("summary -h")
        self.espefuse_py("summary")

    def test_summary_json(self):
        self.espefuse_py("summary --format json")

    def test_get_custom_mac(self):
        self.espefuse_py("get_custom_mac -h")
        if chip_target == "esp32":
            error_msg = None
            ret_code = 0
        else:
            error_msg = "get_custom_mac is not supported!"
            ret_code = 2
        self.espefuse_py("get_custom_mac", check_msg=error_msg, ret_code=ret_code)

    def test_adc_info(self):
        self.espefuse_py("adc_info -h")
        self.espefuse_py("adc_info")


class TestBurnCommands(EfuseTestCase):
    def test_read_protect_efuse(self):
        self.espefuse_py("read_protect_efuse -h")
        if chip_target == "esp32":
            cmd = 'read_protect_efuse \
                   CODING_SCHEME \
                   MAC_VERSION \
                   BLOCK1 \
                   BLOCK2 \
                   BLOCK3'
            count_protects = 5
        else:
            cmd = 'read_protect_efuse \
                   BLOCK_KEY0 \
                   BLOCK_KEY1 \
                   BLOCK_KEY2 \
                   BLOCK_KEY3 \
                   BLOCK_KEY4 \
                   BLOCK_KEY5'
            count_protects = 6
        self.espefuse_py(cmd)
        output = self.espefuse_py(cmd)
        self.assertEqual(count_protects, output.count("is already read protected"))

    def test_read_protect_efuse2(self):
        if chip_target == "esp32":
            self.espefuse_py('write_protect_efuse RD_DIS')
            self.espefuse_py('read_protect_efuse CODING_SCHEME',
                             check_msg='A fatal error occurred: This efuse cannot be read-disabled due the to RD_DIS field is already write-disabled',
                             ret_code=2)
        else:
            self.espefuse_py('write_protect_efuse RD_DIS')
            self.espefuse_py('read_protect_efuse BLOCK_KEY0',
                             check_msg='A fatal error occurred: This efuse cannot be read-disabled due the to RD_DIS field is already write-disabled',
                             ret_code=2)

    def test_write_protect_efuse(self):
        self.espefuse_py("write_protect_efuse -h")
        if chip_target == "esp32":
            efuse_lists = '''WR_DIS RD_DIS CODING_SCHEME CHIP_VERSION CHIP_PACKAGE XPD_SDIO_FORCE
                           XPD_SDIO_REG XPD_SDIO_TIEH SPI_PAD_CONFIG_CLK FLASH_CRYPT_CNT UART_DOWNLOAD_DIS
                           FLASH_CRYPT_CONFIG ADC_VREF BLOCK1 BLOCK2 BLOCK3'''
            efuse_lists2 = 'WR_DIS RD_DIS'
        else:
            efuse_lists = '''RD_DIS DIS_RTC_RAM_BOOT DIS_ICACHE DIS_DOWNLOAD_ICACHE DIS_FORCE_DOWNLOAD
                           DIS_USB DIS_CAN SOFT_DIS_JTAG DIS_DOWNLOAD_MANUAL_ENCRYPT USB_EXCHG_PINS
                           WDT_DELAY_SEL SPI_BOOT_CRYPT_CNT SECURE_BOOT_KEY_REVOKE0
                           SECURE_BOOT_KEY_REVOKE1 SECURE_BOOT_KEY_REVOKE2 KEY_PURPOSE_0 KEY_PURPOSE_1 KEY_PURPOSE_2 KEY_PURPOSE_3 KEY_PURPOSE_4 KEY_PURPOSE_5
                           SECURE_BOOT_EN SECURE_BOOT_AGGRESSIVE_REVOKE FLASH_TPUW DIS_DOWNLOAD_MODE DIS_LEGACY_SPI_BOOT UART_PRINT_CHANNEL
                           DIS_USB_DOWNLOAD_MODE ENABLE_SECURITY_DOWNLOAD UART_PRINT_CONTROL PIN_POWER_SELECTION FLASH_TYPE FORCE_SEND_RESUME SECURE_VERSION
                           MAC SPI_PAD_CONFIG_CLK SPI_PAD_CONFIG_Q SPI_PAD_CONFIG_D SPI_PAD_CONFIG_CS SPI_PAD_CONFIG_HD SPI_PAD_CONFIG_WP SPI_PAD_CONFIG_DQS
                           SPI_PAD_CONFIG_D4 SPI_PAD_CONFIG_D5 SPI_PAD_CONFIG_D6 SPI_PAD_CONFIG_D7 WAFER_VERSION PKG_VERSION BLOCK1_VERSION OPTIONAL_UNIQUE_ID
                           BLOCK2_VERSION BLOCK_USR_DATA BLOCK_KEY0 BLOCK_KEY1 BLOCK_KEY2 BLOCK_KEY3 BLOCK_KEY4 BLOCK_KEY5'''
            efuse_lists2 = 'RD_DIS DIS_RTC_RAM_BOOT'
        self.espefuse_py('write_protect_efuse {}'.format(efuse_lists))
        output = self.espefuse_py('write_protect_efuse {}'.format(efuse_lists2))
        self.assertEqual(2, output.count("is already write protected"))

    def test_write_protect_efuse2(self):
        if chip_target == "esp32":
            self.espefuse_py('write_protect_efuse WR_DIS')
            self.espefuse_py('write_protect_efuse CODING_SCHEME',
                             check_msg='A fatal error occurred: This efuse cannot be write-disabled due to the WR_DIS field is already write-disabled',
                             ret_code=2)

    def test_burn_custom_mac(self):
        self.espefuse_py("burn_custom_mac -h")
        cmd = 'burn_custom_mac AB:CD:EF:11:22:33'
        if chip_target == "esp32":
            self.espefuse_py(cmd, check_msg='Custom MAC Address version 1: ab:cd:ef:11:22:33 (CRC 0x54 OK)')
        else:
            self.espefuse_py(cmd, check_msg='burn_custom_mac is not supported!', ret_code=2)

    def test_burn_custom_mac2(self):
        if chip_target == "esp32":
            self.espefuse_py('burn_custom_mac AB:CD:EF:11:22:33:44',
                             check_msg='A fatal error occurred: MAC Address needs to be a 6-byte hexadecimal format separated by colons (:)!',
                             ret_code=2)

    def test_burn_custom_mac_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py("burn_custom_mac -h")
            self.espefuse_py('burn_custom_mac AB:CD:EF:01:02:03', check_msg='Custom MAC Address version 1: ab:cd:ef:01:02:03 (CRC 0x61 OK)')
            self.espefuse_py('get_custom_mac', check_msg='Custom MAC Address version 1: ab:cd:ef:01:02:03 (CRC 0x61 OK)')

            self.espefuse_py('burn_custom_mac FF:22:33:44:55:66',
                             check_msg='New value contains some bits that cannot be cleared (value will be 0x675745ffefff)',
                             ret_code=2)

    @unittest.skipIf(chip_target == "esp32c3", "TODO: add support set_flash_voltage for ESP32-C3")
    def test_set_flash_voltage_1_8v(self):
        self.espefuse_py("set_flash_voltage -h")
        vdd = "VDD_SDIO" if chip_target == "esp32" else "VDD_SPI"
        self.espefuse_py('set_flash_voltage 1.8V', check_msg='Set internal flash voltage regulator (%s) to 1.8V.' % vdd)
        if chip_target == "esp32":
            error_msg = "A fatal error occurred: Can't set flash regulator to OFF as XPD_SDIO_REG efuse is already burned"
        else:
            error_msg = "A fatal error occurred: Can't set flash regulator to OFF as VDD_SPI_XPD efuse is already burned"
        self.espefuse_py('set_flash_voltage 3.3V', check_msg='Enable internal flash voltage regulator (%s) to 3.3V.' % vdd)
        self.espefuse_py('set_flash_voltage OFF', check_msg=error_msg, ret_code=2)

    @unittest.skipIf(chip_target == "esp32c3", "TODO: add support set_flash_voltage for ESP32-C3")
    def test_set_flash_voltage_3_3v(self):
        vdd = "VDD_SDIO" if chip_target == "esp32" else "VDD_SPI"
        self.espefuse_py('set_flash_voltage 3.3V', check_msg='Enable internal flash voltage regulator (%s) to 3.3V.' % vdd)
        if chip_target == "esp32":
            error_msg = "A fatal error occurred: Can't set regulator to 1.8V is XPD_SDIO_TIEH efuse is already burned"
        else:
            error_msg = "A fatal error occurred: Can't set regulator to 1.8V is VDD_SPI_TIEH efuse is already burned"
        self.espefuse_py('set_flash_voltage 1.8V', check_msg=error_msg, ret_code=2)

        if chip_target == "esp32":
            error_msg = "A fatal error occurred: Can't set flash regulator to OFF as XPD_SDIO_REG efuse is already burned"
        else:
            error_msg = "A fatal error occurred: Can't set flash regulator to OFF as VDD_SPI_XPD efuse is already burned"
        self.espefuse_py('set_flash_voltage OFF', check_msg=error_msg, ret_code=2)

    @unittest.skipIf(chip_target == "esp32c3", "TODO: add support set_flash_voltage for ESP32-C3")
    def test_set_flash_voltage_off(self):
        vdd = "VDD_SDIO" if chip_target == "esp32" else "VDD_SPI"
        self.espefuse_py('set_flash_voltage OFF', check_msg='Disable internal flash voltage regulator (%s)' % vdd)
        self.espefuse_py('set_flash_voltage 3.3V', check_msg='Enable internal flash voltage regulator (%s) to 3.3V.' % vdd)

    @unittest.skipIf(chip_target == "esp32c3", "TODO: add support set_flash_voltage for ESP32-C3")
    def test_set_flash_voltage_off2(self):
        vdd = "VDD_SDIO" if chip_target == "esp32" else "VDD_SPI"
        self.espefuse_py('set_flash_voltage OFF', check_msg='Disable internal flash voltage regulator (%s)' % vdd)
        self.espefuse_py('set_flash_voltage 1.8V', check_msg='Set internal flash voltage regulator (%s) to 1.8V.' % vdd)

    def test_burn_efuse(self):
        self.espefuse_py("burn_efuse -h")
        if chip_target == "esp32":
            self.espefuse_py('burn_efuse \
                              CHIP_VER_REV2 1 \
                              DISABLE_DL_ENCRYPT 1 \
                              CONSOLE_DEBUG_DISABLE 1')
            self.espefuse_py('burn_efuse MAC AB:CD:EF:01:02:03', check_msg="Writing Factory MAC address is not supported", ret_code=2)
            self.espefuse_py('burn_efuse MAC_VERSION 1')
            self.espefuse_py("burn_efuse -h")
            self.espefuse_py('burn_efuse CUSTOM_MAC AB:CD:EF:01:02:03')
            self.espefuse_py('get_custom_mac', check_msg='Custom MAC Address version 1: ab:cd:ef:01:02:03 (CRC 0x61 OK)')
            blk1 = "BLOCK1"
            blk2 = "BLOCK2"
        else:
            self.espefuse_py('burn_efuse \
                              SECURE_BOOT_EN 1 \
                              UART_PRINT_CONTROL 1')
            self.espefuse_py('burn_efuse \
                              OPTIONAL_UNIQUE_ID 0x2328ad5ac9145f698f843a26d6eae168',
                             check_msg="-> 0x2328ad5ac9145f698f843a26d6eae168")
            output = self.espefuse_py('summary -d')
            self.assertIn('read_regs: d6eae168 8f843a26 c9145f69 2328ad5a 00000000 00000000 00000000 00000000', output)
            self.assertIn('= 68 e1 ea d6 26 3a 84 8f 69 5f 14 c9 5a ad 28 23 R/W', output)
            self.espefuse_py('burn_efuse \
                              BLOCK2_VERSION  1',
                             check_msg="Burn into BLOCK_SYS_DATA is forbidden (RS coding scheme does not allow this).",
                             ret_code=2)
            blk1 = "BLOCK_KEY1"
            blk2 = "BLOCK_KEY2"
        output = self.espefuse_py('burn_efuse {} 0x00010203040506070809111111111111111111111111111111110000112233FF'.format(blk1))
        self.assertIn('-> 0x00010203040506070809111111111111111111111111111111110000112233ff', output)
        output = self.espefuse_py('summary -d')
        self.assertIn('read_regs: 112233ff 11110000 11111111 11111111 11111111 08091111 04050607 00010203', output)
        self.assertIn('= ff 33 22 11 00 00 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 09 08 07 06 05 04 03 02 01 00 R/W', output)

        output = self.espefuse_py('burn_efuse {}   00010203040506070809111111111111111111111111111111110000112233FF'.format(blk2))
        self.assertIn('-> 0xff33221100001111111111111111111111111111111109080706050403020100', output)
        output = self.espefuse_py('summary -d')
        self.assertIn('read_regs: 03020100 07060504 11110908 11111111 11111111 11111111 00001111 ff332211', output)
        self.assertIn('= 00 01 02 03 04 05 06 07 08 09 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 11 00 00 11 22 33 ff R/W', output)

    def test_burn_efuse_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py("burn_efuse BLK3_PART_RESERVE 1")
            self.espefuse_py("burn_efuse ADC1_TP_LOW 50")
            self.espefuse_py("burn_efuse ADC1_TP_HIGH 55", check_msg="Burn into BLOCK3 is forbidden (3/4 coding scheme does not allow this)", ret_code=2)

    def test_burn_efuse_with_34_coding_scheme2(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py("burn_efuse BLK3_PART_RESERVE 1")
            self.espefuse_py("burn_efuse \
                              ADC1_TP_LOW 50 \
                              ADC1_TP_HIGH 55 \
                              ADC2_TP_LOW 40 \
                              ADC2_TP_HIGH 45")

    def test_burn_key(self):
        self.espefuse_py("burn_key -h")
        if chip_target == "esp32":
            self.espefuse_py('burn_key BLOCK1 images/efuse/192bit',
                             check_msg="A fatal error occurred: Incorrect key file size 24. Key file must be 32 bytes (256 bits) of raw binary key data.",
                             ret_code=2)
            self.espefuse_py('burn_key \
                              BLOCK1 images/efuse/256bit \
                              BLOCK2 images/efuse/256bit_1 \
                              BLOCK3 images/efuse/256bit_2 --no-protect-key')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit")
            self.check_data_block_in_log(output, "images/efuse/256bit_1")
            self.check_data_block_in_log(output, "images/efuse/256bit_2")

            self.espefuse_py('burn_key \
                              BLOCK1 images/efuse/256bit \
                              BLOCK2 images/efuse/256bit_1 \
                              BLOCK3 images/efuse/256bit_2')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit")
            self.check_data_block_in_log(output, "images/efuse/256bit_1")
            self.check_data_block_in_log(output, "images/efuse/256bit_2")
        else:
            self.espefuse_py('burn_key \
                              BLOCK_KEY0 images/efuse/256bit   XTS_AES_256_KEY_1 \
                              BLOCK_KEY1 images/efuse/256bit_1 XTS_AES_256_KEY_2 \
                              BLOCK_KEY2 images/efuse/256bit_2 XTS_AES_128_KEY   --no-read-protect --no-write-protect')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit", reverse_order=True)
            self.check_data_block_in_log(output, "images/efuse/256bit_1", reverse_order=True)
            self.check_data_block_in_log(output, "images/efuse/256bit_2", reverse_order=True)

            self.espefuse_py('burn_key \
                              BLOCK_KEY0 images/efuse/256bit   XTS_AES_256_KEY_1 \
                              BLOCK_KEY1 images/efuse/256bit_1 XTS_AES_256_KEY_2 \
                              BLOCK_KEY2 images/efuse/256bit_2 XTS_AES_128_KEY')
            output = self.espefuse_py('summary -d')
            self.assertIn('[4 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)
            self.assertIn('[5 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)
            self.assertIn('[6 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)

            self.espefuse_py('burn_key \
                              BLOCK_KEY3 images/efuse/256bit   SECURE_BOOT_DIGEST0 \
                              BLOCK_KEY4 images/efuse/256bit_1 SECURE_BOOT_DIGEST1 \
                              BLOCK_KEY5 images/efuse/256bit_2 SECURE_BOOT_DIGEST2')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit")
            self.check_data_block_in_log(output, "images/efuse/256bit_1")
            self.check_data_block_in_log(output, "images/efuse/256bit_2")

    def test_burn_key_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py('burn_key BLOCK1 images/efuse/256bit',
                             check_msg="A fatal error occurred: Incorrect key file size 32. Key file must be 24 bytes (192 bits) of raw binary key data.",
                             ret_code=2)
            self.espefuse_py('burn_key \
                              BLOCK1 images/efuse/192bit \
                              BLOCK2 images/efuse/192bit_1 \
                              BLOCK3 images/efuse/192bit_2 --no-protect-key')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/192bit")
            self.check_data_block_in_log(output, "images/efuse/192bit_1")
            self.check_data_block_in_log(output, "images/efuse/192bit_2")

            self.espefuse_py('burn_key \
                              BLOCK1 images/efuse/192bit \
                              BLOCK2 images/efuse/192bit_1 \
                              BLOCK3 images/efuse/192bit_2')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/192bit")
            self.check_data_block_in_log(output, "images/efuse/192bit_1")
            self.check_data_block_in_log(output, "images/efuse/192bit_2")

    def test_burn_block_data_check_args(self):
        self.espefuse_py("burn_block_data -h")
        if chip_target == "esp32":
            blk0 = "BLOCK0"
            blk1 = "BLOCK1"
        else:
            blk0 = "BLOCK0"
            blk1 = "BLOCK1"
        self.espefuse_py('burn_block_data \
                          %s images/efuse/224bit \
                          %s' % (blk0, blk1),
                         check_msg="A fatal error occurred: The number of block_name (2) and datafile (1) should be the same.",
                         ret_code=2)

    def test_burn_block_data(self):
        if chip_target == "esp32":
            self.espefuse_py('burn_block_data \
                              BLOCK0 images/efuse/224bit \
                              BLOCK3 images/efuse/256bit')
            output = self.espefuse_py('summary -d')
            self.assertIn('[3 ] read_regs: a3a2a1a0 a7a6a5a4 abaaa9a8 afaeadac b3b2b1b0 b7b6b5b4 bbbab9b8 bfbebdbc', output)
            self.check_data_block_in_log(output, "images/efuse/256bit")

            self.espefuse_py('burn_block_data \
                              BLOCK2 images/efuse/256bit_1')
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/256bit_1")

            self.espefuse_py('burn_block_data \
                              BLOCK1 images/efuse/256bit_2')
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/256bit_2")
        else:
            self.espefuse_py('burn_block_data \
                              BLOCK0 images/efuse/192bit \
                              BLOCK3 images/efuse/256bit')
            output = self.espefuse_py('summary -d')
            self.assertIn('[0 ] read_regs: 00000000 07060500 00000908 00000000 13000000 00161514', output)
            self.assertIn('[3 ] read_regs: a3a2a1a0 a7a6a5a4 abaaa9a8 afaeadac b3b2b1b0 b7b6b5b4 bbbab9b8 bfbebdbc', output)
            self.check_data_block_in_log(output, "images/efuse/256bit")

            self.espefuse_py('burn_block_data \
                              BLOCK10 images/efuse/256bit_1')
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/256bit_1")

            self.espefuse_py('burn_block_data \
                              BLOCK1 images/efuse/192bit \
                              BLOCK5 images/efuse/256bit_1 \
                              BLOCK6 images/efuse/256bit_2')
            output = self.espefuse_py('summary -d')
            self.assertIn('[1 ] read_regs: 00000000 07060500 00000908 00000000 13000000 00161514', output)
            self.check_data_block_in_log(output, "images/efuse/256bit")
            self.check_data_block_in_log(output, "images/efuse/256bit_1", 2)
            self.check_data_block_in_log(output, "images/efuse/256bit_2")

    def test_burn_block_data_with_offset(self):
        if chip_target == "esp32":
            blk0 = "BLOCK0"
            blk1 = "BLOCK1"
            blk2 = "BLOCK2"
            blk3 = "BLOCK3"
        else:
            blk0 = "BLOCK0"
            blk1 = "BLOCK_KEY0"
            blk2 = "BLOCK_KEY1"
            blk3 = "BLOCK_KEY2"
        self.espefuse_py('burn_block_data \
                            --offset 4\
                            %s images/efuse/192bit \
                            %s images/efuse/192bit_1' % (blk1, blk1),
                         check_msg="A fatal error occurred: Found repeated",
                         ret_code=2)
        self.espefuse_py('burn_block_data \
                            --offset 4\
                            %s images/efuse/192bit \
                            %s images/efuse/192bit_1' % (blk1, blk2),
                         check_msg="A fatal error occurred: The 'offset' option is not applicable when a few blocks are passed.",
                         ret_code=2)
        self.espefuse_py('burn_block_data \
                            --offset 9 \
                            %s images/efuse/192bit' % (blk0),
                         check_msg="A fatal error occurred: Data does not fit:",
                         ret_code=2)
        if chip_target == "esp32":
            offset = 1
            self.espefuse_py('burn_block_data --offset %d BLOCK0 images/efuse/192bit' % offset)

        offset = 4
        self.espefuse_py('burn_block_data --offset %d %s images/efuse/192bit_1' % (offset, blk1))
        self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/192bit_1", offset=offset)

        offset = 6
        self.espefuse_py('burn_block_data --offset %d %s images/efuse/192bit_2' % (offset, blk2))
        self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/192bit_2", offset=offset)

        offset = 8
        self.espefuse_py('burn_block_data --offset %d %s images/efuse/192bit_2' % (offset, blk3))
        self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/192bit_2", offset=offset)

    def test_burn_block_data_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py('burn_block_data BLOCK1 images/efuse/256bit',
                             check_msg="A fatal error occurred: Data does not fit: the block1 size is 24 bytes, data file is 32 bytes, offset 0",
                             ret_code=2)

            self.espefuse_py('burn_block_data \
                              BLOCK1 images/efuse/192bit \
                              BLOCK2 images/efuse/192bit_1 \
                              BLOCK3 images/efuse/192bit_2')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/192bit")
            self.check_data_block_in_log(output, "images/efuse/192bit_1")
            self.check_data_block_in_log(output, "images/efuse/192bit_2")

    def test_burn_block_data_with_34_coding_scheme_and_offset(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()

            offset = 4
            self.espefuse_py('burn_block_data --offset %d BLOCK1 images/efuse/128bit' % (offset))
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/128bit", offset=offset)

            offset = 6
            self.espefuse_py('burn_block_data --offset %d BLOCK2 images/efuse/128bit' % (offset))
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/128bit", offset=offset)

            offset = 8
            self.espefuse_py('burn_block_data --offset %d BLOCK3 images/efuse/128bit' % (offset))
            self.check_data_block_in_log(self.espefuse_py('summary -d'), "images/efuse/128bit", offset=offset)

    def test_burn_key_digest(self):
        self.espefuse_py("burn_key_digest -h")
        if chip_target == "esp32":
            esp = self.get_esptool()
            chip_revision = esp.get_chip_description()
            if "revision 3" in chip_revision:
                self.espefuse_py('burn_key_digest secure_images/rsa_secure_boot_signing_key.pem')
                output = self.espefuse_py('summary -d')
                self.assertEqual(1, output.count(" = cb 27 91 a3 71 b0 c0 32 2b f7 37 04 78 ba 09 62 22 4c ab 1c f2 28 78 79 e4 29 67 3e 7d a8 44 63 R/-"))
            else:
                self.espefuse_py('burn_key_digest secure_images/rsa_secure_boot_signing_key.pem',
                                 check_msg="Incorrect chip revision for Secure boot v2.",
                                 ret_code=2)
        else:
            self.espefuse_py('burn_key_digest \
                              BLOCK_KEY0 secure_images/rsa_secure_boot_signing_key.pem SECURE_BOOT_DIGEST0 \
                              BLOCK_KEY1 secure_images/rsa_secure_boot_signing_key2.pem SECURE_BOOT_DIGEST1 \
                              BLOCK_KEY2 ',
                             check_msg="A fatal error occurred: The number of blocks (3), datafile (2) and keypurpose (2) should be the same.",
                             ret_code=2)
            self.espefuse_py('burn_key_digest \
                              BLOCK_KEY0 secure_images/rsa_secure_boot_signing_key.pem SECURE_BOOT_DIGEST0 \
                              BLOCK_KEY1 secure_images/rsa_secure_boot_signing_key2.pem SECURE_BOOT_DIGEST1 \
                              BLOCK_KEY2 secure_images/rsa_secure_boot_signing_key2.pem SECURE_BOOT_DIGEST2')
            output = self.espefuse_py('summary -d')
            self.assertEqual(1, output.count(" = cb 27 91 a3 71 b0 c0 32 2b f7 37 04 78 ba 09 62 22 4c ab 1c f2 28 78 79 e4 29 67 3e 7d a8 44 63 R/-"))
            self.assertEqual(2, output.count(" = 90 1a 74 09 23 8d 52 d4 cb f9 6f 56 3f b3 f4 29 6d ab d6 6a 33 f5 3b 15 ee cd 8c b3 e7 ec 45 d3 R/-"))

    def test_burn_key_from_digest(self):
        #  python espsecure.py digest_rsa_public_key --keyfile test/secure_images/rsa_secure_boot_signing_key.pem \
        #                                            -o secure_images/rsa_public_key_digest.bin
        if chip_target == "esp32":
            self.espefuse_py('burn_key \
                              BLOCK2 secure_images/rsa_public_key_digest.bin --no-protect-key')
            output = self.espefuse_py('summary -d')
            print(output)
            self.assertEqual(1, output.count(" = cb 27 91 a3 71 b0 c0 32 2b f7 37 04 78 ba 09 62 22 4c ab 1c f2 28 78 79 e4 29 67 3e 7d a8 44 63 R/W"))

        elif chip_target == "esp32s2":
            self.espefuse_py('burn_key \
                              BLOCK_KEY0 secure_images/rsa_public_key_digest.bin SECURE_BOOT_DIGEST0')
            output = self.espefuse_py('summary -d')
            self.assertEqual(1, output.count(" = cb 27 91 a3 71 b0 c0 32 2b f7 37 04 78 ba 09 62 22 4c ab 1c f2 28 78 79 e4 29 67 3e 7d a8 44 63 R/-"))

            self.espefuse_py('burn_key_digest \
                              BLOCK_KEY1 secure_images/rsa_secure_boot_signing_key.pem SECURE_BOOT_DIGEST1')
            output = self.espefuse_py('summary -d')
            self.assertEqual(2, output.count(" = cb 27 91 a3 71 b0 c0 32 2b f7 37 04 78 ba 09 62 22 4c ab 1c f2 28 78 79 e4 29 67 3e 7d a8 44 63 R/-"))

    def test_burn_key_digest_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py('burn_key_digest secure_images/rsa_secure_boot_signing_key.pem',
                             check_msg="burn_key_digest only works with 'None' coding scheme",
                             ret_code=2)

    def test_burn_bit(self):
        self.espefuse_py("burn_bit -h")
        if chip_target == "esp32":
            self.espefuse_py('burn_bit BLOCK3 0 1 2 4 8 16 32 64 96 128 160 192 224 255')
            self.espefuse_py('summary', check_msg="17 01 01 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 80")

            self.espefuse_py('burn_bit BLOCK3 3 5 6 7 9 10 11 12 13 14 15 31 63 95 127 159 191 223 254')
            self.espefuse_py('summary', check_msg="ff ff 01 80 01 00 00 80 01 00 00 80 01 00 00 80 01 00 00 80 01 00 00 80 01 00 00 80 01 00 00 c0")
        else:
            self.espefuse_py('burn_bit BLOCK3 0 1 2 4 8 16 32 64 96 128 160 192 224 255')
            self.espefuse_py('summary', check_msg="17 01 01 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 80")
            self.espefuse_py('burn_bit BLOCK3 100', check_msg="Burn into BLOCK_USR_DATA is forbidden (RS coding scheme does not allow this)", ret_code=2)

            self.espefuse_py('burn_bit BLOCK0 13')
            self.espefuse_py('summary', check_msg="[0 ] read_regs: 00002000 00000000 00000000 00000000 00000000 00000000")

            self.espefuse_py('burn_bit BLOCK0 24')
            self.espefuse_py('summary', check_msg="[0 ] read_regs: 01002000 00000000 00000000 00000000 00000000 00000000")

    def test_burn_bit_with_34_coding_scheme(self):
        if chip_target == "esp32":
            self._set_34_coding_scheme()
            self.espefuse_py('burn_bit BLOCK3 0 1 2 4 8 16 32 64 96 128 160 191')
            self.espefuse_py('summary', check_msg="17 01 01 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 00 01 00 00 80")
            self.espefuse_py('burn_bit BLOCK3 17', check_msg="Burn into BLOCK3 is forbidden (3/4 coding scheme does not allow this).", ret_code=2)


class TestByteOrderBurnKeyCommand(EfuseTestCase):
    def test_1_secure_boot_v1(self):
        if chip_target == "esp32":
            self.espefuse_py('burn_key \
                              flash_encryption images/efuse/256bit \
                              secure_boot_v1 images/efuse/256bit_1 --no-protect-key')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit", reverse_order=True)
            self.check_data_block_in_log(output, "images/efuse/256bit_1", reverse_order=True)

            self.espefuse_py('burn_key \
                              flash_encryption  images/efuse/256bit \
                              secure_boot_v1    images/efuse/256bit_1')
            output = self.espefuse_py('summary -d')
            self.assertIn('[1 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)
            self.assertIn('[2 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)
            self.assertIn('[3 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)

    def test_2_secure_boot_v1(self):
        if chip_target == "esp32":
            self.espefuse_py('burn_key \
                              flash_encryption images/efuse/256bit \
                              secure_boot_v2 images/efuse/256bit_1 --no-protect-key')
            output = self.espefuse_py('summary -d')
            self.check_data_block_in_log(output, "images/efuse/256bit", reverse_order=True)
            self.check_data_block_in_log(output, "images/efuse/256bit_1", reverse_order=False)

            self.espefuse_py('burn_key \
                              flash_encryption images/efuse/256bit \
                              secure_boot_v2 images/efuse/256bit_1')
            output = self.espefuse_py('summary -d')
            self.assertIn('[1 ] read_regs: 00000000 00000000 00000000 00000000 00000000 00000000 00000000 00000000', output)
            self.check_data_block_in_log(output, "images/efuse/256bit_1", reverse_order=False)


if __name__ == '__main__':
    if len(sys.argv) > 1:
        chip_target = sys.argv[1]
        if chip_target not in support_list_chips:
            print("Usage: %s - a wrong name of chip" % chip_target)
            sys.exit(1)
        if len(sys.argv) > 3:
            espefuse_port = sys.argv[2]
            reset_port = serial.Serial(sys.argv[3], 115200)
    else:
        chip_target = support_list_chips[0]  # ESP32 by default
    print("HOST_TEST of espefuse.py for %s" % chip_target)

    # unittest also uses argv, so trim the args we used
    sys.argv = [sys.argv[0]] + sys.argv[4:]
    print("Running espefuse.py tests...")
    unittest.main(buffer=True)



#!/usr/bin/env python3

import subprocess as sp

def main(args):
    with open(args.disk, 'rb') as f:
        f.seek(args.block * args.block_size)
        block = (f.read(args.block_size)
            .ljust(args.block_size, b'\xff'))

    # what did you expect?
    print("%-8s  %-s" % ('off', 'data'))
    return sp.run(['xxd', '-g1', '-'], input=block).returncode

if __name__ == "__main__":
    import argparse
    import sys
    parser = argparse.ArgumentParser(
        description="Hex dump a specific block in a disk.")
    parser.add_argument('disk',
        help="File representing the block device.")
    parser.add_argument('block_size', type=lambda x: int(x, 0),
        help="Size of a block in bytes.")
    parser.add_argument('block', type=lambda x: int(x, 0),
        help="Address of block to dump.")
    sys.exit(main(parser.parse_args()))



#!/usr/bin/env python3

import os
import asyncio

# 512 bytes
message = bytearray(512);
bufsize=len(message)
print('message len=', bufsize)

global recv
recv = 0

async def tcp_echo_open (ip, port):
    return await asyncio.open_connection(ip, port)

async def tcp_echo_sender(message, writer):
    print('Writer started')
    while True:
        writer.write(message)
        await writer.drain()

async def tcp_echo_receiver(message, reader):
    global recv
    print('Reader started')
    while True:
        data = ''.encode('utf8')
        while len(data) < bufsize:
            data += await reader.read(bufsize - len(data))
        recv += len(data);
        if data != message:
            print('error')

async def tcp_stat():
    global recv
    dur = 0
    loopsec = 2
    while True:
        last = recv
        await asyncio.sleep(loopsec) # drifting
        dur += loopsec
        print('BW=', (recv - last) * 2 * 8 / 1024 / loopsec, 'Kibits/s avg=', recv * 2 * 8 / 1024 / dur)

loop = asyncio.get_event_loop()
reader, writer = loop.run_until_complete(tcp_echo_open('echo23.local', 23))
loop.create_task(tcp_echo_receiver(message, reader))
loop.create_task(tcp_echo_sender(message, writer))
loop.create_task(tcp_stat())
loop.run_forever()



from mock_decorators import setup, teardown
from threading import Thread
import socket
import time

stop_client_thread = False
client_thread = None

@setup('Simple echo server')
def setup_echo_server(e):
    global stop_client_thread
    global client_thread
    def echo_client_thread():
        time.sleep(1) # let some time for mDNS to start
        server_address = socket.gethostbyname('esp8266-wfs-test.local')
        count = 0
        while count < 5 and not stop_client_thread:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            sock.connect((server_address, 5000))
            sock.settimeout(1.0)
            buf = 'a' * 1023 + '\n'
            sock.sendall(bytes(buf.encode('utf-8')))
            data = ''
            retries = 0
            while len(data) < 1024 and retries < 3:
                data += sock.recv(1024).decode('utf-8')
                retries += 1
            print('Received {} bytes'.format(len(data)))
            if len(data) != 1024:
                raise RuntimeError('client failed to receive response')
            count += 1
        
    stop_client_thread = False
    client_thread = Thread(target=echo_client_thread)
    client_thread.start()

@teardown('Simple echo server')
def teardown_echo_server(e):
    global stop_client_thread
    stop_client_thread = True
    client_thread.join()

    



#!/usr/bin/env python
# This file includes the operations with eFuses for ESP32S2 chip
#
# Copyright (C) 2020 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

import argparse

import espsecure

import esptool

from . import fields
from .. import util
from ..base_operations import (add_common_commands, add_force_write_always, burn_bit, burn_block_data, burn_efuse, dump,  # noqa: F401
                               read_protect_efuse, summary, write_protect_efuse)  # noqa: F401


def protect_options(p):
    p.add_argument('--no-write-protect', help='Disable write-protecting of the key. The key remains writable. '
                   '(The keys use the RS coding scheme that does not support post-write data changes. Forced write can damage RS encoding bits.)'
                   ' The write-protecting of keypurposes does not depend on the option, it will be set anyway.', action='store_true')
    p.add_argument('--no-read-protect', help='Disable read-protecting of the key. The key remains readable software.'
                   'The key with keypurpose[USER, RESERVED and *_DIGEST] will remain readable anyway. '
                   'For the rest keypurposes the read-protection will be defined the option (Read-protect by default).', action='store_true')


def add_commands(subparsers, efuses):
    add_common_commands(subparsers, efuses)
    burn_key = subparsers.add_parser('burn_key', help='Burn the key block with the specified name')
    protect_options(burn_key)
    add_force_write_always(burn_key)
    burn_key.add_argument('block', help='Key block to burn', action='append', choices=efuses.BLOCKS_FOR_KEYS)
    burn_key.add_argument('keyfile', help='File containing 256 bits of binary key data', action='append', type=argparse.FileType('rb'))
    burn_key.add_argument('keypurpose', help='Purpose to set.', action='append', choices=fields.EfuseKeyPurposeField.KEY_PURPOSES_NAME)
    for _ in efuses.BLOCKS_FOR_KEYS:
        burn_key.add_argument('block', help='Key block to burn', nargs="?", action='append', metavar="BLOCK", choices=efuses.BLOCKS_FOR_KEYS)
        burn_key.add_argument('keyfile', help='File containing 256 bits of binary key data', nargs="?", action='append', metavar="KEYFILE",
                              type=argparse.FileType('rb'))
        burn_key.add_argument('keypurpose', help='Purpose to set.', nargs="?", action='append', metavar="KEYPURPOSE",
                              choices=fields.EfuseKeyPurposeField.KEY_PURPOSES_NAME)

    burn_key_digest = subparsers.add_parser('burn_key_digest', help='Parse a RSA public key and burn the digest to key efuse block')
    protect_options(burn_key_digest)
    add_force_write_always(burn_key_digest)
    burn_key_digest.add_argument('block', help='Key block to burn', action='append', choices=efuses.BLOCKS_FOR_KEYS)
    burn_key_digest.add_argument('keyfile', help='Key file to digest (PEM format)', action='append', type=argparse.FileType('rb'))
    burn_key_digest.add_argument('keypurpose', help='Purpose to set.', action='append', choices=fields.EfuseKeyPurposeField.DIGEST_KEY_PURPOSES)
    for _ in efuses.BLOCKS_FOR_KEYS:
        burn_key_digest.add_argument('block', help='Key block to burn', nargs="?", action='append', metavar="BLOCK", choices=efuses.BLOCKS_FOR_KEYS)
        burn_key_digest.add_argument('keyfile', help='Key file to digest (PEM format)', nargs="?", action='append', metavar="KEYFILE",
                                     type=argparse.FileType('rb'))
        burn_key_digest.add_argument('keypurpose', help='Purpose to set.', nargs="?", action='append', metavar="KEYPURPOSE",
                                     choices=fields.EfuseKeyPurposeField.DIGEST_KEY_PURPOSES)

    p = subparsers.add_parser('set_flash_voltage',
                              help='Permanently set the internal flash voltage regulator to either 1.8V, 3.3V or OFF. '
                              'This means GPIO45 can be high or low at reset without changing the flash voltage.')
    p.add_argument('voltage', help='Voltage selection', choices=['1.8V', '3.3V', 'OFF'])

    p = subparsers.add_parser('burn_custom_mac', help='Not supported! Burn a 48-bit Custom MAC Address to EFUSE is.')
    p.add_argument('mac', help='Custom MAC Address to burn given in hexadecimal format with bytes separated by colons'
                   ' (e.g. AB:CD:EF:01:02:03).', nargs="?")
    add_force_write_always(p)

    p = subparsers.add_parser('get_custom_mac', help='Not supported! Prints the Custom MAC Address.')


def burn_custom_mac(esp, efuses, args):
    raise esptool.FatalError("burn_custom_mac is not supported!")


def get_custom_mac(esp, efuses, args):
    raise esptool.FatalError("get_custom_mac is not supported!")


def set_flash_voltage(esp, efuses, args):
    sdio_force = efuses["VDD_SPI_FORCE"]
    sdio_tieh = efuses["VDD_SPI_TIEH"]
    sdio_reg = efuses["VDD_SPI_XPD"]

    # check efuses aren't burned in a way which makes this impossible
    if args.voltage == 'OFF' and sdio_reg.get() != 0:
        raise esptool.FatalError("Can't set flash regulator to OFF as VDD_SPI_XPD efuse is already burned")

    if args.voltage == '1.8V' and sdio_tieh.get() != 0:
        raise esptool.FatalError("Can't set regulator to 1.8V is VDD_SPI_TIEH efuse is already burned")

    if args.voltage == 'OFF':
        msg = """
Disable internal flash voltage regulator (VDD_SPI). SPI flash will need to be powered from an external source.
The following efuse is burned: VDD_SPI_FORCE.
It is possible to later re-enable the internal regulator (%s) by burning an additional efuse
""" % ("to 3.3V" if sdio_tieh.get() != 0 else "to 1.8V or 3.3V")
    elif args.voltage == '1.8V':
        msg = """
Set internal flash voltage regulator (VDD_SPI) to 1.8V.
The following efuses are burned: VDD_SPI_FORCE, VDD_SPI_XPD.
It is possible to later increase the voltage to 3.3V (permanently) by burning additional efuse VDD_SPI_TIEH
"""
    elif args.voltage == '3.3V':
        msg = """
Enable internal flash voltage regulator (VDD_SPI) to 3.3V.
The following efuses are burned: VDD_SPI_FORCE, VDD_SPI_XPD, VDD_SPI_TIEH.
"""
    print(msg)

    sdio_force.save(1)   # Disable GPIO45
    if args.voltage != 'OFF':
        sdio_reg.save(1)  # Enable internal regulator
    if args.voltage == '3.3V':
        sdio_tieh.save(1)
    print("VDD_SPI setting complete.")
    efuses.burn_all()


def adc_info(esp, efuses, args):
    print("")
    if efuses["BLOCK2_VERSION"].get() == 1:
        print("Temperature Sensor Calibration = {}C".format(efuses["TEMP_SENSOR_CAL"].get()))

        print("")
        print("ADC1 readings stored in efuse BLOCK2:")
        print("    MODE0 D1 reading  (250mV):  {}".format(efuses["ADC1_MODE0_D1"].get()))
        print("    MODE0 D2 reading  (600mV):  {}".format(efuses["ADC1_MODE0_D2"].get()))

        print("    MODE1 D1 reading  (250mV):  {}".format(efuses["ADC1_MODE1_D1"].get()))
        print("    MODE1 D2 reading  (800mV):  {}".format(efuses["ADC1_MODE1_D2"].get()))

        print("    MODE2 D1 reading  (250mV):  {}".format(efuses["ADC1_MODE2_D1"].get()))
        print("    MODE2 D2 reading  (1000mV): {}".format(efuses["ADC1_MODE2_D2"].get()))

        print("    MODE3 D1 reading  (250mV):  {}".format(efuses["ADC1_MODE3_D1"].get()))
        print("    MODE3 D2 reading  (2000mV): {}".format(efuses["ADC1_MODE3_D2"].get()))

        print("")
        print("ADC2 readings stored in efuse BLOCK2:")
        print("    MODE0 D1 reading  (250mV):  {}".format(efuses["ADC2_MODE0_D1"].get()))
        print("    MODE0 D2 reading  (600mV):  {}".format(efuses["ADC2_MODE0_D2"].get()))

        print("    MODE1 D1 reading  (250mV):  {}".format(efuses["ADC2_MODE1_D1"].get()))
        print("    MODE1 D2 reading  (800mV):  {}".format(efuses["ADC2_MODE1_D2"].get()))

        print("    MODE2 D1 reading  (250mV):  {}".format(efuses["ADC2_MODE2_D1"].get()))
        print("    MODE2 D2 reading  (1000mV): {}".format(efuses["ADC2_MODE2_D2"].get()))

        print("    MODE3 D1 reading  (250mV):  {}".format(efuses["ADC2_MODE3_D1"].get()))
        print("    MODE3 D2 reading  (2000mV): {}".format(efuses["ADC2_MODE3_D2"].get()))
    else:
        print("BLOCK2_VERSION = {}".format(efuses["BLOCK2_VERSION"].get_meaning()))


def burn_key(esp, efuses, args, digest=None):
    if digest is None:
        datafile_list = args.keyfile[0:len([name for name in args.keyfile if name is not None]):]
    else:
        datafile_list = digest[0:len([name for name in digest if name is not None]):]
    efuses.force_write_always = args.force_write_always
    block_name_list = args.block[0:len([name for name in args.block if name is not None]):]
    keypurpose_list = args.keypurpose[0:len([name for name in args.keypurpose if name is not None]):]

    util.check_duplicate_name_in_list(block_name_list)
    if len(block_name_list) != len(datafile_list) or len(block_name_list) != len(keypurpose_list):
        raise esptool.FatalError("The number of blocks (%d), datafile (%d) and keypurpose (%d) should be the same." %
                                 (len(block_name_list), len(datafile_list), len(keypurpose_list)))

    print("Burn keys to blocks:")
    for block_name, datafile, keypurpose in zip(block_name_list, datafile_list, keypurpose_list):
        efuse = None
        for block in efuses.blocks:
            if block_name == block.name or block_name in block.alias:
                efuse = efuses[block.name]
        if efuse is None:
            raise esptool.FatalError("Unknown block name - %s" % (block_name))
        num_bytes = efuse.bit_len // 8

        block_num = efuses.get_index_block_by_name(block_name)
        block = efuses.blocks[block_num]

        if digest is None:
            data = datafile.read()
        else:
            data = datafile

        print(" - %s" % (efuse.name), end=" ")
        revers_msg = None
        if efuses[block.key_purpose_name].need_reverse(keypurpose):
            revers_msg = "\tReversing byte order for AES-XTS hardware peripheral"
            data = data[::-1]
        print("-> [%s]" % (util.hexify(data, " ")))
        if revers_msg:
            print(revers_msg)
        if len(data) != num_bytes:
            raise esptool.FatalError("Incorrect key file size %d. Key file must be %d bytes (%d bits) of raw binary key data." %
                                     (len(data), num_bytes, num_bytes * 8))

        if efuses[block.key_purpose_name].need_rd_protect(keypurpose):
            read_protect = False if args.no_read_protect else True
        else:
            read_protect = False
        write_protect = not args.no_write_protect

        # using efuse instead of a block gives the advantage of checking it as the whole field.
        efuse.save(data)

        disable_wr_protect_key_purpose = False
        if efuses[block.key_purpose_name].get() != keypurpose:
            if efuses[block.key_purpose_name].is_writeable():
                print("\t'%s': '%s' -> '%s'." % (block.key_purpose_name, efuses[block.key_purpose_name].get(), keypurpose))
                efuses[block.key_purpose_name].save(keypurpose)
                disable_wr_protect_key_purpose = True
            else:
                raise esptool.FatalError("It is not possible to change '%s' to '%s' because write protection bit is set." %
                                         (block.key_purpose_name, keypurpose))
        else:
            print("\t'%s' is already '%s'." % (block.key_purpose_name, keypurpose))
            if efuses[block.key_purpose_name].is_writeable():
                disable_wr_protect_key_purpose = True

        if disable_wr_protect_key_purpose:
            print("\tDisabling write to '%s'." % block.key_purpose_name)
            efuses[block.key_purpose_name].disable_write()

        if read_protect:
            print("\tDisabling read to key block")
            efuse.disable_read()

        if write_protect:
            print("\tDisabling write to key block")
            efuse.disable_write()
        print("")

    if not write_protect:
        print("Keys will remain writeable (due to --no-write-protect)")
    if args.no_read_protect:
        print("Keys will remain readable (due to --no-read-protect)")

    efuses.burn_all()
    print("Successful")


def burn_key_digest(esp, efuses, args):
    digest_list = []
    datafile_list = args.keyfile[0:len([name for name in args.keyfile if name is not None]):]
    block_list = args.block[0:len([block for block in args.block if block is not None]):]
    for block_name, datafile in zip(block_list, datafile_list):
        efuse = None
        for block in efuses.blocks:
            if block_name == block.name or block_name in block.alias:
                efuse = efuses[block.name]
        if efuse is None:
            raise esptool.FatalError("Unknown block name - %s" % (block_name))
        num_bytes = efuse.bit_len // 8
        digest = espsecure._digest_rsa_public_key(datafile)
        if len(digest) != num_bytes:
            raise esptool.FatalError("Incorrect digest size %d. Digest must be %d bytes (%d bits) of raw binary key data." %
                                     (len(digest), num_bytes, num_bytes * 8))
        digest_list.append(digest)
    burn_key(esp, efuses, args, digest=digest_list)



#!/usr/bin/env python3

import struct
import sys
import json
import io
import itertools as it
from readmdir import Tag, MetadataPair

def main(args):
    superblock = None
    gstate = b'\0\0\0\0\0\0\0\0\0\0\0\0'
    dirs = []
    mdirs = []
    corrupted = []
    cycle = False
    with open(args.disk, 'rb') as f:
        tail = (args.block1, args.block2)
        hard = False
        while True:
            for m in it.chain((m for d in dirs for m in d), mdirs):
                if set(m.blocks) == set(tail):
                    # cycle detected
                    cycle = m.blocks
            if cycle:
                break

            # load mdir
            data = []
            blocks = {}
            for block in tail:
                f.seek(block * args.block_size)
                data.append(f.read(args.block_size)
                    .ljust(args.block_size, b'\xff'))
                blocks[id(data[-1])] = block

            mdir = MetadataPair(data)
            mdir.blocks = tuple(blocks[id(p.data)] for p in mdir.pair)

            # fetch some key metadata as a we scan
            try:
                mdir.tail = mdir[Tag('tail', 0, 0)]
                if mdir.tail.size != 8 or mdir.tail.data == 8*b'\xff':
                    mdir.tail = None
            except KeyError:
                mdir.tail = None

            # have superblock?
            try:
                nsuperblock = mdir[
                    Tag(0x7ff, 0x3ff, 0), Tag('superblock', 0, 0)]
                superblock = nsuperblock, mdir[Tag('inlinestruct', 0, 0)]
            except KeyError:
                pass

            # have gstate?
            try:
                ngstate = mdir[Tag('movestate', 0, 0)]
                gstate = bytes((a or 0) ^ (b or 0)
                    for a,b in it.zip_longest(gstate, ngstate.data))
            except KeyError:
                pass

            # corrupted?
            if not mdir:
                corrupted.append(mdir)

            # add to directories
            mdirs.append(mdir)
            if mdir.tail is None or not mdir.tail.is_('hardtail'):
                dirs.append(mdirs)
                mdirs = []

            if mdir.tail is None:
                break

            tail = struct.unpack('<II', mdir.tail.data)
            hard = mdir.tail.is_('hardtail')

    # find paths
    dirtable = {}
    for dir in dirs:
        dirtable[frozenset(dir[0].blocks)] = dir

    pending = [("/", dirs[0])]
    while pending:
        path, dir = pending.pop(0)
        for mdir in dir:
            for tag in mdir.tags:
                if tag.is_('dir'):
                    try:
                        npath = tag.data.decode('utf8')
                        dirstruct = mdir[Tag('dirstruct', tag.id, 0)]
                        nblocks = struct.unpack('<II', dirstruct.data)
                        nmdir = dirtable[frozenset(nblocks)]
                        pending.append(((path + '/' + npath), nmdir))
                    except KeyError:
                        pass

        dir[0].path = path.replace('//', '/')

    # print littlefs + version info
    version = ('?', '?')
    if superblock:
        version = tuple(reversed(
            struct.unpack('<HH', superblock[1].data[0:4].ljust(4, b'\xff'))))
    print("%-47s%s" % ("littlefs v%s.%s" % version,
        "data (truncated, if it fits)"
        if not any([args.no_truncate, args.log, args.all]) else ""))

    # print gstate
    print("gstate 0x%s" % ''.join('%02x' % c for c in gstate))
    tag = Tag(struct.unpack('<I', gstate[0:4].ljust(4, b'\xff'))[0])
    blocks = struct.unpack('<II', gstate[4:4+8].ljust(8, b'\xff'))
    if tag.size or not tag.isvalid:
        print("  orphans >=%d" % max(tag.size, 1))
    if tag.type:
        print("  move dir {%#x, %#x} id %d" % (
            blocks[0], blocks[1], tag.id))

    # print mdir info
    for i, dir in enumerate(dirs):
        print("dir %s" % (json.dumps(dir[0].path)
            if hasattr(dir[0], 'path') else '(orphan)'))

        for j, mdir in enumerate(dir):
            print("mdir {%#x, %#x} rev %d (was %d)%s%s" % (
                mdir.blocks[0], mdir.blocks[1], mdir.rev, mdir.pair[1].rev,
                ' (corrupted!)' if not mdir else '',
                ' -> {%#x, %#x}' % struct.unpack('<II', mdir.tail.data)
                if mdir.tail else ''))

            f = io.StringIO()
            if args.log:
                mdir.dump_log(f, truncate=not args.no_truncate)
            elif args.all:
                mdir.dump_all(f, truncate=not args.no_truncate)
            else:
                mdir.dump_tags(f, truncate=not args.no_truncate)

            lines = list(filter(None, f.getvalue().split('\n')))
            for k, line in enumerate(lines):
                print("%s %s" % (
                    ' ' if j == len(dir)-1 else
                    'v' if k == len(lines)-1 else
                    '|',
                    line))

    errcode = 0
    for mdir in corrupted:
        errcode = errcode or 1
        print("*** corrupted mdir {%#x, %#x}! ***" % (
            mdir.blocks[0], mdir.blocks[1]))

    if cycle:
        errcode = errcode or 2
        print("*** cycle detected {%#x, %#x}! ***" % (
            cycle[0], cycle[1]))

    return errcode

if __name__ == "__main__":
    import argparse
    import sys
    parser = argparse.ArgumentParser(
        description="Dump semantic info about the metadata tree in littlefs")
    parser.add_argument('disk',
        help="File representing the block device.")
    parser.add_argument('block_size', type=lambda x: int(x, 0),
        help="Size of a block in bytes.")
    parser.add_argument('block1', nargs='?', default=0,
        type=lambda x: int(x, 0),
        help="Optional first block address for finding the superblock.")
    parser.add_argument('block2', nargs='?', default=1,
        type=lambda x: int(x, 0),
        help="Optional second block address for finding the superblock.")
    parser.add_argument('-l', '--log', action='store_true',
        help="Show tags in log.")
    parser.add_argument('-a', '--all', action='store_true',
        help="Show all tags in log, included tags in corrupted commits.")
    parser.add_argument('-T', '--no-truncate', action='store_true',
        help="Show the full contents of files/attrs/tags.")
    sys.exit(main(parser.parse_args()))



#!/usr/bin/env python
# This file describes eFuses controller for ESP32-S2 chip
#
# Copyright (C) 2020 Espressif Systems (Shanghai) PTE LTD
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301 USA.
from __future__ import division, print_function

import reedsolo

from .mem_definition import EfuseDefineBlocks, EfuseDefineFields, EfuseDefineRegisters
from ..emulate_efuse_controller_base import EmulateEfuseControllerBase, FatalError


class EmulateEfuseController(EmulateEfuseControllerBase):
    """ The class for virtual ESP32 operation. Using for HOST_TEST.
    """
    CHIP_NAME = "ESP32-S2"
    mem = None
    debug = False
    Blocks  = EfuseDefineBlocks
    Fields  = EfuseDefineFields
    REGS    = EfuseDefineRegisters

    def __init__(self, efuse_file=None, debug=False):
        super(EmulateEfuseController, self).__init__(efuse_file, debug)
        self.write_reg(self.REGS.EFUSE_STATUS_REG, 1)

    """ esptool method start >>"""

    def get_chip_description(self):
        return ""

    def get_crystal_freq(self):
        return 40  # MHz (common for all chips)

    """ << esptool method end """

    def handle_writing_event(self, addr, value):
        if addr == self.REGS.EFUSE_CMD_REG:
            if value & self.REGS.EFUSE_PGM_CMD:
                self.copy_blocks_wr_regs_to_rd_regs(updated_block=(value >> 2) & 0xF)
                self.clean_blocks_wr_regs()
                self.check_rd_protection_area()
                self.write_reg(addr, 0)
                self.write_reg(self.REGS.EFUSE_STATUS_REG, 1)
            elif value == self.REGS.EFUSE_READ_CMD:
                self.write_reg(addr, 0)
                self.write_reg(self.REGS.EFUSE_STATUS_REG, 1)
                self.save_to_file()

    def get_bitlen_of_block(self, blk, wr=False):
        if blk.id == 0:
            if wr:
                return 32 * 8
            else:
                return 32 * blk.len
        else:
            if wr:
                rs_coding = 32 * 3
                return 32 * 8 + rs_coding
            else:
                return 32 * blk.len

    def handle_coding_scheme(self, blk, data):
        if blk.id != 0:
            # CODING_SCHEME RS applied only for all blocks except BLK0.
            coded_bytes = 12
            data.pos = coded_bytes * 8
            plain_data = data.readlist('32*uint:8')[::-1]
            # takes 32 bytes
            # apply RS encoding
            rs = reedsolo.RSCodec(coded_bytes)
            # 32 byte of data + 12 bytes RS
            calc_encoded_data = list(rs.encode([x for x in plain_data]))
            data.pos = 0
            if calc_encoded_data != data.readlist('44*uint:8')[::-1]:
                raise FatalError("Error in coding scheme data")
            data = data[coded_bytes * 8:]
        if blk.len < 8:
            data = data[(8 - blk.len) * 32:]
        return data



#!/usr/bin/env python3

import re
import sys

PATTERN = ['LFS_ASSERT', 'assert']
PREFIX = 'LFS'
MAXWIDTH = 16

ASSERT = "__{PREFIX}_ASSERT_{TYPE}_{COMP}"
FAIL = """
__attribute__((unused))
static void __{prefix}_assert_fail_{type}(
        const char *file, int line, const char *comp,
        {ctype} lh, size_t lsize,
        {ctype} rh, size_t rsize) {{
    printf("%s:%d:assert: assert failed with ", file, line);
    __{prefix}_assert_print_{type}(lh, lsize);
    printf(", expected %s ", comp);
    __{prefix}_assert_print_{type}(rh, rsize);
    printf("\\n");
    fflush(NULL);
    raise(SIGABRT);
}}
"""

COMP = {
    '==': 'eq',
    '!=': 'ne',
    '<=': 'le',
    '>=': 'ge',
    '<':  'lt',
    '>':  'gt',
}

TYPE = {
    'int': {
        'ctype': 'intmax_t',
        'fail': FAIL,
        'print': """
        __attribute__((unused))
        static void __{prefix}_assert_print_{type}({ctype} v, size_t size) {{
            (void)size;
            printf("%"PRIiMAX, v);
        }}
        """,
        'assert': """
        #define __{PREFIX}_ASSERT_{TYPE}_{COMP}(file, line, lh, rh)
        do {{
            __typeof__(lh) _lh = lh;
            __typeof__(lh) _rh = (__typeof__(lh))rh;
            if (!(_lh {op} _rh)) {{
                __{prefix}_assert_fail_{type}(file, line, "{comp}",
                        (intmax_t)_lh, 0, (intmax_t)_rh, 0);
            }}
        }} while (0)
        """
    },
    'bool': {
        'ctype': 'bool',
        'fail': FAIL,
        'print': """
        __attribute__((unused))
        static void __{prefix}_assert_print_{type}({ctype} v, size_t size) {{
            (void)size;
            printf("%s", v ? "true" : "false");
        }}
        """,
        'assert': """
        #define __{PREFIX}_ASSERT_{TYPE}_{COMP}(file, line, lh, rh)
        do {{
            bool _lh = !!(lh);
            bool _rh = !!(rh);
            if (!(_lh {op} _rh)) {{
                __{prefix}_assert_fail_{type}(file, line, "{comp}",
                        _lh, 0, _rh, 0);
            }}
        }} while (0)
        """
    },
    'mem': {
        'ctype': 'const void *',
        'fail': FAIL,
        'print': """
        __attribute__((unused))
        static void __{prefix}_assert_print_{type}({ctype} v, size_t size) {{
            const uint8_t *s = v;
            printf("\\\"");
            for (size_t i = 0; i < size && i < {maxwidth}; i++) {{
                if (s[i] >= ' ' && s[i] <= '~') {{
                    printf("%c", s[i]);
                }} else {{
                    printf("\\\\x%02x", s[i]);
                }}
            }}
            if (size > {maxwidth}) {{
                printf("...");
            }}
            printf("\\\"");
        }}
        """,
        'assert': """
        #define __{PREFIX}_ASSERT_{TYPE}_{COMP}(file, line, lh, rh, size)
        do {{
            const void *_lh = lh;
            const void *_rh = rh;
            if (!(memcmp(_lh, _rh, size) {op} 0)) {{
                __{prefix}_assert_fail_{type}(file, line, "{comp}",
                        _lh, size, _rh, size);
            }}
        }} while (0)
        """
    },
    'str': {
        'ctype': 'const char *',
        'fail': FAIL,
        'print': """
        __attribute__((unused))
        static void __{prefix}_assert_print_{type}({ctype} v, size_t size) {{
            __{prefix}_assert_print_mem(v, size);
        }}
        """,
        'assert': """
        #define __{PREFIX}_ASSERT_{TYPE}_{COMP}(file, line, lh, rh)
        do {{
            const char *_lh = lh;
            const char *_rh = rh;
            if (!(strcmp(_lh, _rh) {op} 0)) {{
                __{prefix}_assert_fail_{type}(file, line, "{comp}",
                        _lh, strlen(_lh), _rh, strlen(_rh));
            }}
        }} while (0)
        """
    }
}

def mkdecls(outf, maxwidth=16):
    outf.write("#include <stdio.h>\n")
    outf.write("#include <stdbool.h>\n")
    outf.write("#include <stdint.h>\n")
    outf.write("#include <inttypes.h>\n")
    outf.write("#include <signal.h>\n")

    for type, desc in sorted(TYPE.items()):
        format = {
            'type': type.lower(), 'TYPE': type.upper(),
            'ctype': desc['ctype'],
            'prefix': PREFIX.lower(), 'PREFIX': PREFIX.upper(),
            'maxwidth': maxwidth,
        }
        outf.write(re.sub('\s+', ' ',
            desc['print'].strip().format(**format))+'\n')
        outf.write(re.sub('\s+', ' ',
            desc['fail'].strip().format(**format))+'\n')

        for op, comp in sorted(COMP.items()):
            format.update({
                'comp': comp.lower(), 'COMP': comp.upper(),
                'op': op,
            })
            outf.write(re.sub('\s+', ' ',
                desc['assert'].strip().format(**format))+'\n')

def mkassert(type, comp, lh, rh, size=None):
    format = {
        'type': type.lower(), 'TYPE': type.upper(),
        'comp': comp.lower(), 'COMP': comp.upper(),
        'prefix': PREFIX.lower(), 'PREFIX': PREFIX.upper(),
        'lh': lh.strip(' '),
        'rh': rh.strip(' '),
        'size': size,
    }
    if size:
        return ((ASSERT + '(__FILE__, __LINE__, {lh}, {rh}, {size})')
            .format(**format))
    else:
        return ((ASSERT + '(__FILE__, __LINE__, {lh}, {rh})')
            .format(**format))


# simple recursive descent parser
LEX = {
    'ws':       [r'(?:\s|\n|#.*?\n|//.*?\n|/\*.*?\*/)+'],
    'assert':   PATTERN,
    'string':   [r'"(?:\\.|[^"])*"', r"'(?:\\.|[^'])\'"],
    'arrow':    ['=>'],
    'paren':    ['\(', '\)'],
    'op':       ['strcmp', 'memcmp', '->'],
    'comp':     ['==', '!=', '<=', '>=', '<', '>'],
    'logic':    ['\&\&', '\|\|'],
    'sep':      [':', ';', '\{', '\}', ','],
}

class ParseFailure(Exception):
    def __init__(self, expected, found):
        self.expected = expected
        self.found = found

    def __str__(self):
        return "expected %r, found %s..." % (
            self.expected, repr(self.found)[:70])

class Parse:
    def __init__(self, inf, lexemes):
        p = '|'.join('(?P<%s>%s)' % (n, '|'.join(l))
            for n, l in lexemes.items())
        p = re.compile(p, re.DOTALL)
        data = inf.read()
        tokens = []
        while True:
            m = p.search(data)
            if m:
                if m.start() > 0:
                    tokens.append((None, data[:m.start()]))
                tokens.append((m.lastgroup, m.group()))
                data = data[m.end():]
            else:
                tokens.append((None, data))
                break
        self.tokens = tokens
        self.off = 0

    def lookahead(self, *pattern):
        if self.off < len(self.tokens):
            token = self.tokens[self.off]
            if token[0] in pattern or token[1] in pattern:
                self.m = token[1]
                return self.m
        self.m = None
        return self.m

    def accept(self, *patterns):
        m = self.lookahead(*patterns)
        if m is not None:
            self.off += 1
        return m

    def expect(self, *patterns):
        m = self.accept(*patterns)
        if not m:
            raise ParseFailure(patterns, self.tokens[self.off:])
        return m

    def push(self):
        return self.off

    def pop(self, state):
        self.off = state

def passert(p):
    def pastr(p):
        p.expect('assert') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        p.expect('strcmp') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        lh = pexpr(p) ; p.accept('ws')
        p.expect(',') ; p.accept('ws')
        rh = pexpr(p) ; p.accept('ws')
        p.expect(')') ; p.accept('ws')
        comp = p.expect('comp') ; p.accept('ws')
        p.expect('0') ; p.accept('ws')
        p.expect(')')
        return mkassert('str', COMP[comp], lh, rh)

    def pamem(p):
        p.expect('assert') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        p.expect('memcmp') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        lh = pexpr(p) ; p.accept('ws')
        p.expect(',') ; p.accept('ws')
        rh = pexpr(p) ; p.accept('ws')
        p.expect(',') ; p.accept('ws')
        size = pexpr(p) ; p.accept('ws')
        p.expect(')') ; p.accept('ws')
        comp = p.expect('comp') ; p.accept('ws')
        p.expect('0') ; p.accept('ws')
        p.expect(')')
        return mkassert('mem', COMP[comp], lh, rh, size)

    def paint(p):
        p.expect('assert') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        lh = pexpr(p) ; p.accept('ws')
        comp = p.expect('comp') ; p.accept('ws')
        rh = pexpr(p) ; p.accept('ws')
        p.expect(')')
        return mkassert('int', COMP[comp], lh, rh)

    def pabool(p):
        p.expect('assert') ; p.accept('ws') ; p.expect('(') ; p.accept('ws')
        lh = pexprs(p) ; p.accept('ws')
        p.expect(')')
        return mkassert('bool', 'eq', lh, 'true')

    def pa(p):
        return p.expect('assert')

    state = p.push()
    lastf = None
    for pa in [pastr, pamem, paint, pabool, pa]:
        try:
            return pa(p)
        except ParseFailure as f:
            p.pop(state)
            lastf = f
    else:
        raise lastf

def pexpr(p):
    res = []
    while True:
        if p.accept('('):
            res.append(p.m)
            while True:
                res.append(pexprs(p))
                if p.accept('sep'):
                    res.append(p.m)
                else:
                    break
            res.append(p.expect(')'))
        elif p.lookahead('assert'):
            res.append(passert(p))
        elif p.accept('assert', 'ws', 'string', 'op', None):
            res.append(p.m)
        else:
            return ''.join(res)

def pexprs(p):
    res = []
    while True:
        res.append(pexpr(p))
        if p.accept('comp', 'logic', ','):
            res.append(p.m)
        else:
            return ''.join(res)

def pstmt(p):
    ws = p.accept('ws') or ''
    lh = pexprs(p)
    if p.accept('=>'):
        rh = pexprs(p)
        return ws + mkassert('int', 'eq', lh, rh)
    else:
        return ws + lh


def main(args):
    inf = open(args.input, 'r') if args.input else sys.stdin
    outf = open(args.output, 'w') if args.output else sys.stdout

    lexemes = LEX.copy()
    if args.pattern:
        lexemes['assert'] = args.pattern
    p = Parse(inf, lexemes)

    # write extra verbose asserts
    mkdecls(outf, maxwidth=args.maxwidth)
    if args.input:
        outf.write("#line %d \"%s\"\n" % (1, args.input))

    # parse and write out stmt at a time
    try:
        while True:
            outf.write(pstmt(p))
            if p.accept('sep'):
                outf.write(p.m)
            else:
                break
    except ParseFailure as f:
        pass

    for i in range(p.off, len(p.tokens)):
        outf.write(p.tokens[i][1])

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(
        description="Cpp step that increases assert verbosity")
    parser.add_argument('input', nargs='?',
        help="Input C file after cpp.")
    parser.add_argument('-o', '--output', required=True,
        help="Output C file.")
    parser.add_argument('-p', '--pattern', action='append',
        help="Patterns to search for starting an assert statement.")
    parser.add_argument('--maxwidth', default=MAXWIDTH, type=int,
        help="Maximum number of characters to display for strcmp and memcmp.")
    main(parser.parse_args())



# Cloned from https://github.com/miki725/xunitmerge
# to fix a Python3 error.
#
# xunitmerge is MIT licensed by Miroslav Shubernetskiy https://github.com/miki725
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from contextlib import contextmanager
from xml.etree import ElementTree as etree
from xml.sax.saxutils import quoteattr

import six


CNAME_TAGS = ('system-out', 'skipped', 'error', 'failure')
CNAME_PATTERN = '<![CDATA[{}]]>'
TAG_PATTERN = '<{tag}{attrs}>{text}</{tag}>'


@contextmanager
def patch_etree_cname(etree):
    """
    Patch ElementTree's _serialize_xml function so that it will
    write text as CDATA tag for tags tags defined in CNAME_TAGS.

    >>> import re
    >>> from xml.etree import ElementTree
    >>> xml_string = '''
    ... <testsuite name="nosetests" tests="1" errors="0" failures="0" skip="0">
    ...     <testcase classname="some.class.Foo" name="test_system_out" time="0.001">
    ...         <system-out>Some output here</system-out>
    ...     </testcase>
    ...     <testcase classname="some.class.Foo" name="test_skipped" time="0.001">
    ...         <skipped type="unittest.case.SkipTest" message="Skipped">Skipped</skipped>
    ...     </testcase>
    ...     <testcase classname="some.class.Foo" name="test_error" time="0.001">
    ...         <error type="KeyError" message="Error here">Error here</error>
    ...     </testcase>
    ...     <testcase classname="some.class.Foo" name="test_failure" time="0.001">
    ...         <failure type="AssertionError" message="Failure here">Failure here</failure>
    ...     </testcase>
    ... </testsuite>
    ... '''
    >>> tree = ElementTree.fromstring(xml_string)
    >>> with patch_etree_cname(ElementTree):
    ...    saved = str(ElementTree.tostring(tree))
    >>> systemout = re.findall(r'(<system-out>.*?</system-out>)', saved)[0]
    >>> print(systemout)
    <system-out><![CDATA[Some output here]]></system-out>
    >>> skipped = re.findall(r'(<skipped.*?</skipped>)', saved)[0]
    >>> print(skipped)
    <skipped message="Skipped" type="unittest.case.SkipTest"><![CDATA[Skipped]]></skipped>
    >>> error = re.findall(r'(<error.*?</error>)', saved)[0]
    >>> print(error)
    <error message="Error here" type="KeyError"><![CDATA[Error here]]></error>
    >>> failure = re.findall(r'(<failure.*?</failure>)', saved)[0]
    >>> print(failure)
    <failure message="Failure here" type="AssertionError"><![CDATA[Failure here]]></failure>
    """
    original_serialize = etree._serialize_xml

    def _serialize_xml(write, elem, *args, **kwargs):
        if elem.tag in CNAME_TAGS:
            attrs = ' '.join(
                ['{}={}'.format(k, quoteattr(v))
                 for k, v in sorted(elem.attrib.items())]
            )
            attrs = ' ' + attrs if attrs else ''
            text = CNAME_PATTERN.format(elem.text)
            write(TAG_PATTERN.format(
                tag=elem.tag,
                attrs=attrs,
                text=text
            ))
        else:
            original_serialize(write, elem, *args, **kwargs)

    etree._serialize_xml = etree._serialize['xml'] = _serialize_xml

    yield

    etree._serialize_xml = etree._serialize['xml'] = original_serialize


def merge_trees(*trees):
    """
    Merge all given XUnit ElementTrees into a single ElementTree.
    This combines all of the children test-cases and also merges
    all of the metadata of how many tests were executed, etc.
    """
    first_tree = trees[0]
    first_root = first_tree.getroot()

    if len(trees) == 0:
        return first_tree

    for tree in trees[1:]:
        root = tree.getroot()

        # append children elements (testcases)
        first_root.extend(root.getchildren())

        # combine root attributes which stores the number
        # of executed tests, skipped tests, etc
        for key, value in first_root.attrib.items():
            if not value.isdigit():
                continue
            combined = six.text_type(int(value) + int(root.attrib.get(key, '0')))
            first_root.set(key, combined)

    return first_tree


def merge_xunit(files, output, callback=None):
    """
    Merge the given xunit xml files into a single output xml file.

    If callback is not None, it will be called with the merged ElementTree
    before the output file is written (useful for applying other fixes to
    the merged file). This can either modify the element tree in place (and
    return None) or return a completely new ElementTree to be written.
    """
    trees = []

    for f in files:
        trees.append(etree.parse(f))

    merged = merge_trees(*trees)

    if callback is not None:
        result = callback(merged)
        if result is not None:
            merged = result

    with patch_etree_cname(etree):
        merged.write(output, encoding='utf-8', xml_declaration=True)



#!/usr/bin/python3

# This script pulls the list of Mozilla trusted certificate authorities
# from the web at the "mozurl" below, parses the file to grab the PEM
# for each cert, and then generates DER files in a new ./data directory
# Upload these to a SPIFFS filesystem and use the CertManager to parse
# and use them for your outgoing SSL connections.
#
# Script by Earle F. Philhower, III.  Released to the public domain.
from __future__ import print_function
import csv
import os
import sys
from subprocess import Popen, PIPE, call
try:
    from urllib.request import urlopen
except:
    from urllib2 import urlopen
try:
    from StringIO import StringIO
except:
    from io import StringIO

# Mozilla's URL for the CSV file with included PEM certs
mozurl = "https://ccadb-public.secure.force.com/mozilla/IncludedCACertificateReportPEMCSV"

# Load the manes[] and pems[] array from the URL
names = []
pems = []
response = urlopen(mozurl)
csvData = response.read()
if sys.version_info[0] > 2:
    csvData = csvData.decode('utf-8')
csvFile = StringIO(csvData)
csvReader = csv.reader(csvFile)
for row in csvReader:
    names.append(row[0]+":"+row[1]+":"+row[2])
    pems.append(row[30])
del names[0] # Remove headers
del pems[0] # Remove headers

# Try and make ./data, skip if present
try:
    os.mkdir("data")
except:
    pass

derFiles = []
idx = 0
# Process the text PEM using openssl into DER files
for i in range(0, len(pems)):
    certName = "data/ca_%03d.der" % (idx);
    thisPem = pems[i].replace("'", "")
    print(names[i] + " -> " + certName)
    ssl = Popen(['openssl','x509','-inform','PEM','-outform','DER','-out', certName], shell = False, stdin = PIPE)
    pipe = ssl.stdin
    pipe.write(thisPem.encode('utf-8'))
    pipe.close()
    ssl.wait()
    if os.path.exists(certName):
        derFiles.append(certName)
        idx = idx + 1

if os.path.exists("data/certs.ar"):
    os.unlink("data/certs.ar");

arCmd = ['ar', 'q', 'data/certs.ar'] + derFiles;
call( arCmd )

for der in derFiles:
    os.unlink(der)
